function [t,cml] = chainstates(x,par,model)

% [t,cml] = chainstates(x,par,model)
% Restauration step via MAP algorithm          
% input:
%	x	data (cell) {N} x [nk,d]
%	par     parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.A   	transition matrix [k,k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix : 'full','utriangular' or 
%               'udiagonal'
%       -.fam   family of conditionnal law : 'normal' or 'exp'
% output:
%       t    =   hidden states   (cell) {N} x [nk]
%       cml  =   classification loglikelihood   
%
% Example : 
%
% p =
%
%    0.4000    0.6000
%                       
% mu =
%
%    -1    -1    -1
%     1     1     1 
%
% S(:,:,1) =
%
%     1     0     0
%     0     1     0
%     0     0     1
%
%
% S(:,:,2) =
%
%    0.5000         0         0
%         0    0.5000         0
%         0         0    0.5000
%                            
% par=struct('A',A,'p',p,'mu',mu,'S',S);
%
% [x,z] = chainxrnd([10,30,25,22,23,43,21], par);
%
% [t,cml] = chainstates(x,par);
%
%  cml =
%
%  -36.9183 -112.8846  -99.1094  -77.5765  -91.9968 -173.6808  -93.7459  
%
%  any(t{1}-z{1})
%
%  ans =
%
%     0

k = length(par.p);		% number of states
N = length(x);                  % sample size

% conditional probabilities
% ----------------------------------

[f,logf] = chainxpdf(x,par,model);			   % density
for n = 1:N
	if (~all(f{n}))
		t = [];
		cml = -Inf;
		fprintf(1,' At least one element of sample has conditionnal probability zero');
		return
	end	
end
    
% classification step
% -------------------

if ((all(all(par.A))) & all(par.p))
	% viterbilog should be more accurate than viterbi
	% but requires A > 0 and p > 0
	[t,cml] = viterbilog(logf,par);     % map
else
   	if all(par.p)
		par.A = par.A + realmin * ones(size(par.A));
		for i = 1:size(par.A,1)
			par.A(i,:) = par.A(i,:)./ sum(par.A(i,:));
		end
		[t,cml] = viterbilog(logf,par);
	end
	if all(all(par.A))
		par.p = par.p + realmin * ones(size(par.p));
		par.p = par.p / sum(par.p);
		[t,cml] = viterbilog(logf,par);
	end
	if (~all(all(par.A)) & ~all(par.p))
		par.A = par.A + realmin * ones(size(par.A));
		for i = 1:size(par.A,1)
			par.A(i,:) = par.A(i,:)./ sum(par.A(i,:));
		end
		par.p = par.p + realmin * ones(size(par.p));
		par.p = par.p / sum(par.p);
		[t,cml] = viterbilog(logf,par);
	end
end
