function [par,xml,nbit] = chainxem1(x,parinit,parknown,model,algo,cvg,maxit,cutsem,sempar)

% [par,xml,nbit] = chainxem1(x,parinit,parknown,algo,cvg,maxit)
% One algorithm xem in the normal case: em, cem or sem.
% input:
%	x	data (cell) {N} x [nk,d]
%	parinit	initial parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%       -.O     observation matrix ('np' family) [k,v]
%	-.A   	transition matrix [k,k]
%	parknown	known parameters (structure)
%	-.p	'unknown' or proportions [k]
%	-.mu	'unknown' or centers/means [k,d]
%	-.S	'unknown' or variance matrices [d,d,k]
%       -.O     'unknown' or observation matrix ('np' family) [k,v]
%	-.A   	'unknown' or transition matrix [k,k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix 
%       -.fam   family of conditionnal law : 'normal' or 'exp'
%	algo	algorithm 'em', 'cem', 'gibbs', 
%               'sem' or any combination or them (cell)
%	cvg	convergence 'xml', 'maxit' or 'xmlORmaxit'
%	maxit	maximum iterations number
%	cutsem	canceled first iterations number for sem
%	sempar	parameters type from sem
% output:
%	par	estimated parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%       -.O     observation matrix ('np' family) [k,v]
%	-.A 	transition matrix [k,k]
%	xml	loglikelihood (em and sem) or 
%               classification  loglikelihood (cem)
%	nbit	used iteration number
% note:	par is empty if no convergence
%
% Example : 
%
% x=cell(1,1);
% x =  
%     {[]}  
%
% x{1}=[-1.2,3;1.3,5.4;-01,-0.5];     
% x = 
%     [3x2 double]  
%
%  p=[0.368938, 0.243353, 0.387709]; 
% p =  
%     0.3689    0.2434    0.3877     
%
%    S=zeros(2,2,3);
% 
% S(:,:,1) =
%
%      0     0
%      0     0
%
% 
% S(:,:,2) =
%
%      0     0
%      0     0
%
%
% S(:,:,3) =
%
%      0     0
%      0     0
%
% S(:,:,1)=[0.767118,0.204696;0.204696 , 0.954253];
%
% S(:,:,2)=[0.678661,0.551735;0.551735,0.689453];
%
% S(:,:,3)=[0.647363, 0.219151;0.219151, 0.229258]
% 
% S(:,:,1) =
% 
%     0.7671    0.2047
%     0.2047    0.9543
%
%
% S(:,:,2) =
% 
%     0.6787    0.5517
%     0.5517    0.6895
%
%
% S(:,:,3) =
% 
%     0.6474    0.2192
%     0.2192    0.2293
%
% mu=[0.846233,0.663049;0.222531,0.823503;0.115978,0.787944];
%
% mu =
%
%     0.8462    0.6630
%     0.2225    0.8235
%     0.1160    0.7879
%
%  A=[0.524478,0.183443,0.292079;
%     0.205076,0.400017,0.394908;
%     0.148128,0.0618789,0.789993];
%
% A =
%
%     0.5245    0.1834    0.2921
%     0.2051    0.4000    0.3949
%     0.1481    0.0619    0.7900    
%
%  par=struct('p',{p},'mu',{mu},'A',{A},'S',{S})
% 
% par =
% 
%      p: [0.3689 0.2434 0.3877]
%     mu: [3x2 double]
%      A: [3x3 double]
%      S: [2x2x3 double]                   

k = length(parinit.p);		% number of clusters
N = length(x);                  % sample size
epscvg = 1e-6;			% epsilon for convergence

% modify convergence mode
% -----------------------

switch algo
  case 'sem', cvg = 'maxit';		% sem: only 'maxit' possible
  case 'em'			% em: 'partition' is replaced by 'xml'
    if strcmp(cvg,'partition')
      cvg = 'xml';
    end
  case 'gibbs'
    if strcmp(cvg,'partition')
      cvg = 'xml';
    end
end

fprintf(1,'[%s',algo);

% initializations
% ---------------

cvgok = 0;			% convergence flag
nbit = 0;			% iteration number
told = NaN;			% partition
xmlold = -Inf;			% criterion

par = parinit;			% par set to initial par
if ~strcmp(parknown.p,'unknown'),
  par.p = parknown.p; end % par set to known par
if ~strcmp(parknown.mu,'unknown'), par.mu = parknown.mu; end
if ~strcmp(parknown.S,'unknown'), par.S = parknown.S; end
if ~strcmp(parknown.O,'unknown'), par.O = parknown.O; end
if ~strcmp(parknown.A,'unknown'), par.A = parknown.A; end

if strcmp(model.fam,'normal')
     % badly conditioned matrix: exit algorithm
     % ------------------------------------------

     if Sbadcond(par.S)
         par = []; xml = [];
	 fprintf(1,'.badly conditioned variance matrix');
     return
     end
end

if strcmp(algo,'gibbs')
     zold = cell(N);
     for n = 1:N
         zold{n} = zeros(size(x{n},1),k);
     end
     [f,logf] = chainxpdf(x,par,model);
     zold = chaingibbsinit(par,f);
end

sumpar.p = 0 * par.p;	 % sum of all parameters for sem
sumpar.mu = 0 * par.mu;
sumpar.O = 0 * par.O;
sumpar.A = 0 * par.A;

% iterations
% ----------

while ~cvgok

  nbit = nbit + 1;		% one more iteration

  if ~mod(nbit,100)
    fprintf(1,'.%d',nbit);	% display every 100 iteration number
  end

  % step E : conditional probabilities
  % ----------------------------------

  [f,logf] = chainxpdf(x,par,model);			   % density
       for n = 1:N
            if (~all(f{n}))
                par = [];
		xml = -Inf;
		fprintf(1,' At least one element of sample has conditionnal probability zero');
		return
	    end	
	end


    switch algo   
    
    % cem: step C 
    % ----------- 
 
    case 'cem', 

        if all(par.A)
           % viterbilog should be more accurate than viterbi
           % but requires A > 0
           [t,ml] = viterbilog(logf,par);     % map
	else 
	   [t,ml] = viterbi(f,par);
        end;

        [z_zsuiv,z] = chainlab2bin(t,k);   % binary labels

    % sem: step S 
    % ----------- 
 
    case 'sem', 
      
      [condalpha,probx] = condforward(f,par);        % forward algorithm
                                              % useful to calculate probx
      ml = chainml(probx);                           % loglikelihood of par
                                              % (caution: before random z) 

      condbeta = condbackward(probx,f,par);          % backward algorithm

      [z_zsuiv,z] = chainlab2bin(chainrndlabel(condbeta,f,exp(ml),par),k);

    % gibbs: step S 
    % ------------- 
 
    case 'gibbs', 

	
      [condalpha,probx] = condforward(f,par);        % forward algorithm
                                              % useful to calculate ml
      ml = chainml(probx);                           % loglikelihood of par
                                              % (caution: before random z) 

	[z_zsuiv,z] = chainlab2bin(chaingibbslabel(par,f,zold),k);
        for n = 1:N
	     zold{n} = bin2lab(z{n});
        end
 
    % em: no specific step 
    % -------------------- 
 
    case 'em', 

      [condalpha,probx] = condforward(f,par);         % forward algorithm
      ml = chainml(probx);                           % likelihood of par
      condbeta = condbackward(probx,f,par);          % backward algorithm

      [z_zsuiv,z] = mcondprob(condalpha,condbeta,probx,par,f);
                   % conditional probabilities
   
 
 
    otherwise par = []; xml = []; fprintf(1,'unknown algorithm'); return 
 
  end 

  xml = sum(ml);
 
  % empty cluster: exit algorithm 
  % ----------------------------- 
 
%  nk = sum(t,1)'; 
%  if emptycla(nk), par = []; xml = []; fprintf(1,'.empty cluster'); return,
% end   
  % step M: update parameters 
  % ------------------------- 
 
  if strcmp(parknown.mu,'unknown') & ...
	(strcmp(model.fam,'normal') | strcmp(model.fam,'exp'))
    [totalz,par.mu] = estimuc(x,z);		% centers if unknown 
    % fprintf(1,'centers estimated \n');
        if isempty(par.mu)
             par = []; xml = [];   
             fprintf(1,'can''t estimate centers\n');
             return 
        end
   elseif (strcmp(model.fam,'normal') | strcmp(model.fam,'exp')) % isfield(par,'mu')
         par.mu = parknown.mu;
         d = size(x{1},2);	% dimension of data

         totalmu = zeros(k,d,N);	
         totalz = zeros(k,N);	
         % totalmu ./  totalz = estimated centers for each element in sample

         for n = 1:N			% loop on sample
                  nk = size(z{n},1);	% length of nth sequence
                  totalz(:,n) = (z{n})' * ones(nk,1);
         end;

         totalz = sum(totalz,2);

         if ~all(totalz)
                  fprintf(1,'\n at least one state unused; can''t estimate centers\n');
                 par.mu = [];
		 par = []; xml = [];  
	  end
     else
         par.mu = 'none';	
     end
		 
  if strcmp(parknown.O,'unknown') & strcmp(model.fam,'np')
    	
	[totalz,par.O] = estimOc(x,z,size(par.O,2));
	% observation matrices if unknown
        if isempty(par.O)
             par = []; xml = [];   
             fprintf(1,'can''t estimate observation matrix\n');
             return 
        end
   elseif strcmp(model.fam,'np')
         par.O = parknown.O;
         d = 1;	% dimension of data
         
         totalz = sum(z,2);

         if ~all(totalz)
                  fprintf(1,'\n at least one state unused; can''t estimate observation matrix\n');
                 par.O = [];
		 par = []; xml = [];  
	  end
     end

     if (strcmp(parknown.S,'unknown') ...
     	 & ~strcmp(model.fam,'np'))
	% for models with exponential distribution, 
	% parameters can also be estimated from variance
	
  	par.S = estimSc(x,par.mu,z,model.model,totalz,par.S);	
	% variance matrices if unknown       
	if isempty(par.S)
             par = []; xml = [];   
             fprintf(1,'can''t estimate covariance matrices\n');
             return 
        end
    end 

    if strcmp(parknown.A,'unknown') 
       par.A = estimAc(z,z_zsuiv);
       % transition matrix if unknown 
       if isempty(par.A)
             par = []; xml = [];   
             fprintf(1,'can''t estimate transition matrix\n');
             return 
       end
  end   
  if strcmp(parknown.p,'unknown') 
       par.p = estimpc(z,model.model,par.A);
       % proportions if unknown        
       if isempty(par.p)
             par = []; xml = [];   
             fprintf(1,'can''t estimate proportions\n');
             return   
       end
  end   

  if strcmp(model.fam,'normal')

     % badly conditioned matrix: exit algorithm 
     % ------------------------------------------ 
	     
     if Sbadcond(par.S) 
         par = []; xml = []; 
	 fprintf(1,'.badly conditioned variance matrix\n'); 
	 return 
     end 
  end

  % save parameters sum or max for sem 
  % ---------------------------------- 
 
  if strcmp(algo,'sem') 
	if strcmp(sempar,'mean') & (nbit > cutsem)	
		% sempar is mean and enough iterations       

		sumpar.p = sumpar.p + par.p;		% proportions      
		if isfiel(par,'mu')
			sumpar.mu = sumpar.mu + par.mu;		
			% centers  
		end
		if strcmp(model.fam,'normal')
                     sumpar.S = sumpar.S + par.S;	
		     % variance matrices for normal family
		end
		if isfield(par,'O')
			sumpar.O = sumpar.O + par.O;		
			% observation matrix
		end
		sumpar.A = sumpar.A + par.A;
	
	elseif strcmp(sempar,'max') 	 % sempar is max  
	      if xml > xmlold 
        	parmax.p = par.p;			% proportions 
		if isfield(par,'mu')
	        	parmax.mu = par.mu;		% centers 
		end
		if isfield(par,'S')
			parmax.S = par.S;		% variance matrices 
		end
		if isfield(par,'O')
	        	parmax.O = par.O;		% centers 
		end
 		parmax.A = par.A;
      		end 
	end 
  end 
 
  % convergence ? 
  % ------------- 
 
  switch cvg 
 
    case 'maxit', cvgok = (nbit >= maxit);	% stop with iteration number 
    case 'xml',				% stop with criterion stability 
      cvgok = (abs(xml-xmlold) <= epscvg) ;	% xml stability ? 
      xmlold = xml; 
    case 'xmlORmaxit'			% stop with xml stability or maxit 
      cvgok = (nbit >= maxit) | (abs(xml-xmlold) <= epscvg); 
      xmlold = xml;     
    otherwise par = []; xml = []; fprintf(1,'.unknown convergence'); return 
 
  end 
 
  cvgok = cvgok | (k == 1);			% if one cluster, cvg is on 
 
end % while ~cvgok 

%  in the sem case, mean of the (nbit-cutsem) previous paremeters 
%  if sempar is 'mean' 
%                , parameters value of the max xml if sempar is 'max'  
%----------------------------------------------------------------------------------   
if strcmp(algo,'sem') & (k > 1) 
  switch sempar 
 
    case 'mean'				% mean of the sem parameters 
      if cutsem >= nbit			% cutsem must be less than maxit 
        par = [];				% if not: exit the algorithm 
        xml = []; 
        fprintf(1,'.cutsem must be less than maxit'); 
        return 
      end 
	par.p = sumpar.p / (nbit - cutsem);	% mean of proportions 
	if isfield(par,'mu')
      		par.mu = sumpar.mu / (nbit - cutsem);	
		% mean of centers 
	end
	if strcmp(model.fam,'normal')
            par.S = sumpar.S / (nbit - cutsem);	
	    % mean of variance matrices for normal family
	end
	if isfield(par,'O')
      		par.O = sumpar.O / (nbit - cutsem);	
		% mean of observation matrices
	end
	par.A = sumpar.A / (nbit - cutsem);   	% mean of transition matrices
 
    case 'max'				% parameters of the max xml of sem 
	par.p = parmax.p;			% proportions 
	if isfield(par,'mu')
      		par.mu = parmax.mu;		% centers 
	end
	if isfield(par,'S')
      		par.S = parmax.S;		% variance matrices 
	end
	if isfield(par,'O')
      		par.O = parmax.O;		% observation matrix
	end
	par.A = parmax.A;			% transition matrix
    otherwise par = []; xml = []; fprintf(1,'.unknown sempar'); return 
 
  end 
 
	[condalpha,probx] = condforward(f,par);     % forward algorithm
	ml = chainml(probx);    	
	xml = sum(ml);				    % criterion computation  
end 
 
% display the iteration number 
% ---------------------------- 
 
if mod(nbit,100) 
  fprintf(1,'.%d',nbit);			% iteration number 
end 
fprintf(1,']'); 
 
