function [f,logf] = chainxpdf(x,par,model)

% [f,logf] = chainxpdf(x,par,model)
% Normal components and mixture density.
% input:
%	x	data (cell) {N} x [nk,d]
%	par	parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.invS	inverse of S [d,d,k] OPTIONAL
%	-.detS	determinant of S [k] OPTIONAL
%       -.O     observation matrix ('np' family) [k,v]
%	-.A	transition matrix [k,k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix : 'full','utriangular' or 
%               'udiagonal'
%       -.fam   family of conditionnal law : 'normal', 'exp' or 'np'
% output:
%	f	components density {N} x [nk,k]
%	logf	log components density {N} x [nk,k]
%

% initializations
% ---------------

N = length(x);		        % sample size 
if isfield(par,'mu')
   d = length(par.mu(1,:));	% sample size and dimension space
else
   d = 1;
end
k = length(par.p);		% number of clusters

% for normal model
% ----------------

if strcmp(model.fam,'normal')
     % if invS is not provided: compute it
     % -----------------------------------

     if ~isfield(par,'invS')
         par.invS = par.S;
	 for ik = 1:k
              par.invS(:,:,ik) = inv(par.S(:,:,ik));	
              % inverse of the kth mat. var.
	 end
     end

	 % if detS is not provided: compute it
	 % -----------------------------------

     if ~isfield(par,'detS')
         par.detS = par.p;
	 for ik = 1:k
	      par.detS(ik) = det(par.S(:,:,ik));  % det. of the kth mat. var.
	 end
     end

     % logf {N} x [nk,k]
     % -----------------

     logf = cell(N,1);

     for n = 1:N       				% loop on the elements of
						% sample

	nk = size(x{n},1);			% length on the sequence

	logf{n} = zeros(nk,k);

	for ik = 1:k

  		% center the sample: x{n} - muk
  		xminusmuk = x{n} - ones(nk,1) * par.mu(ik,:);

 		 % fast calculus of -0.5 * xminusmuk' * inv(S) * xminusmuk
  		logf{n}(:,ik) =  ...
		-0.5 * sum((xminusmuk * par.invS(:,:,ik)).* xminusmuk,2);

  		% log density of the component ik
  		logf{n}(:,ik) =   ... 
		logf{n}(:,ik) -d/2*log(2*pi) - 0.5*log(par.detS(ik));
	end

	% f {N} x [nk,k] 
	% --------------

	f{n} = exp(logf{n});
    end % for n

% for exponential model
% ---------------------
     elseif strcmp(model.fam,'exp')

     % logf {N} x [nk,k]
     % -----------------

     logf = cell(N,1);

     for n = 1:N       				% loop on the elements of
						% sample

	nk = size(x{n},1);			% length on the sequence

	logf{n} = zeros(nk,k);

	for ik = 1:k  		
  		logf{n}(:,ik) =  (x{n} / (-1 * par.mu(ik))) ...
                               - log(par.mu(ik)) * ones(nk,1);
	end

	% f {N} x [nk,k] 
	% --------------

	f{n} = exp(logf{n});
    end % for n

% for non parametric model
% ---------------------

     elseif strcmp(model.fam,'np')

     	% logf {N} x [nk,k]
	% -----------------

     	logf = cell(N,1);

     	for n = 1:N       				% loop on the elements of
							% sample
		% logf{n} = zeros(nk,k);

  		logf{n} =  log(par.O(:,x{n}))';
         
          	% f [N,k] 
          	% -------

        	f{n} = exp(logf{n});

     	end % for n

    else
       fprintf(1,'unknown family');
       f{n} = zeros(size(x{n},1),k);
    end % if


