function [x,z] = chainxrnd(seq_length,par,model)

% [x,z] = chainxrnd(seq_length,par,model)
% A n-sample simulated from the normal mixture parameters.
% input:
%	seq_length 	lenth of sequences [N]
%	par		parameters (structure)
%	-.p		proportions [k]
%	-.mu		centers [k,d]
%	-.S		variance matrices [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A		transition matrix [k,k]
%	model	        model and number of clusters (structure)
%	-.model	        model: 'plkCk' or value cell of strings [nbmod]
%	-.k	        number of clusters: n^0.3 or value [nbk]
%       -.A             type of transition matrix : 'full','utriangular' or 
%                       'udiagonal'
%       -.fam           family of conditionnal law : 'normal', 'exp' or 'np'
% output:
%	x	        data (cell) {N} x [nk,d]
%	z	        states {N} x [nk]
%
% Example:
%
% k =
%
%     2     
%
% d = 
%
%     3
%
% p =
%
%    0.2000    0.8000
%
%
% mu =
%
%     0     0     0
%     1    -1     1   
%   
% S=zeros(d,d,k);
%
% for i=1:k       
% S(:,:,i)=i*eye(d,d)    
% end
%
% S(:,:,1) =
%
%     1     0     0
%     0     1     0
%     0     0     1
%
%
% S(:,:,2) =
%
%     2     0     0
%     0     2     0
%     0     0     2     
%
% A=[0.3, 0.7; 0.9, 0.1];
%
% A =
% 
%     0.3000    0.7000
%     0.9000    0.1000  
%                               
% par=struct('p',p,'mu',mu,'S',S,'A',A) 
%
% par =
%
%      p: [0.2000 0.8000]
%     mu: [2x3 double]
%      S: [3x3x2 double]
%      A: [2x2 double]
%                          
%
% model.fam = 'normal';
%
% model = 
% 
%     fam: 'normal'
%
% [x,z] = chainxrnd([6], par, model)
%
% x{1} =
%
%   -0.3313    0.5049    1.7069
%   -1.1746   -0.2454    1.2781
%   -0.4441   -3.1461    0.2253
%   -0.4017    0.0097    0.2608
%    1.2456   -0.8991    0.9814
%   -0.1161    0.3165   -0.5803    
%   
% z{1} =
%
%     2
%     1
%     2
%     1
%     2
%     1

if ~strcmp(model.fam,'np')
	[k,d] = size(par.mu);	% number of clusters and space dimension
else
	k = size(par.O, 1);
	d = 1;
end
N = length(seq_length);	% size of sample

for n = 1:N		% loop on sample

	nk = seq_length(n);		% length of nth sequence
	x{n} = randn(nk,d);	% nk points in Rd from N(0,I)

	% generate nk values in dimension 1 from a U(0,1) distribution
	c = rand(nk,1);		% nk real values from U(0,1)

	% choose point origin
	% -------------------

	z{n} = zeros(nk,1);		% hidden states
	sump = 0;
	ik = 0;
	while ((c(1,1) >=  sump) & (ik <= k))
		ik = ik + 1;
		sump = sump + par.p(ik);
		z{n}(1,1) = ik;
	end;

	% Markov chain
	% ------------

	for t = 2: nk
		sump = 0;
		ik = 0;
		while ((c(t,1) >=  sump) & (ik <= k))
                        ik = ik + 1;
                        sump = sump + par.A(z{n}(t-1,1),ik);
                        z{n}(t,1) = ik;
		end;
	end;

	if ~strcmp(model.fam,'np')
		clear c;
	end

% linear transformation of x in order to come from the mixture
% ------------------------------------------------------------

        for ik = 1:k
	
		% all x from the ikth cluster
  		
            	whereisk = (z{n} == ik);
	    	% indices of the observations being in state ik

	    	if strcmp(model.fam,'normal')
		    	x{n}(whereisk,:) = x{n}(whereisk,:) * ...
                     	   sqrtm(par.S(:,:,ik)) ...
                           + ones(sum(whereisk),1) * par.mu(ik,:);

            	elseif strcmp(model.fam,'exp')
		    	x{n}(whereisk,:) = exprnd(par.mu(ik),sum(whereisk),1);
	    	else
		    	% model.fam is supposed to be 'np'
			v = size(par.O, 2);
			sump = 0;
			c = rand(nk, 1);
			for iv = 1:v
				whereisv = (c >= sump);
				x{n}(whereisk & whereisv,:) = ...
				   iv .* ones(sum(whereisk & whereisv),1);
  				sump = sump + par.O(ik, iv);
			end
            	end
         end % for ik
end
