function argout = normixarg(argin)

% argout = normixarg(argin)
% Control arguments of normxem.
% input:
%	argin	user arguments
% output:
%	argout	transformed arguments

% make structure array
% --------------------

if ~isempty(argin)				% some arguments are present
  nameargout = argin(1:2:end);		% get names of arguments
  valargout = argin(2:2:end);			% get values of arguments
  argout = cell2struct(valargout,nameargout,2);	% structure array
else
  argout = {};
end

% data
% ----

if ~isfield(argout,'x')
  error('data x needed')
end
[n,d] = size(argout.x);	% sample size and space dimension

% zknown default value
% --------------------

if ~isfield(argout,'zknown')
  argout.zknown = 'unknown';
end

% parknown default value 
% ----------------------

if ~isfield(argout,'parknown') 	% all structure not exist
  argout.parknown = [];
end
if ~isfield(argout.parknown,'p')	% proportions not defined
  argout.parknown.p = 'unknown';
end
if ~isfield(argout.parknown,'mu')	% centers not defined
  argout.parknown.mu = 'unknown';
end
if ~isfield(argout.parknown,'O')	% observation matrix 
  argout.parknown.O = 'unknown';	% not defined
end
if ~isfield(argout.parknown,'S')
  argout.parknown.S = 'unknown';
end

% init default value
% ------------------

if isfield(argout,'parinit') | ...		% parinit has priority on zinit
   (~isfield(argout,'parinit') & ~isfield(argout,'zinit'))
  argout.init = 'parameters';
else 
  argout.init = 'partition';
end

% zinit default value
% -------------------

if ~isfield(argout,'zinit')
  argout.zinit = 'random';
end

% parinit default value
% ---------------------

if ~isfield(argout,'parinit')		% all structure not exist
  argout.parinit = [];
end
if ~isfield(argout.parinit,'p')	% proportions not defined
  argout.parinit.p = 'equal';
end
if ~isfield(argout.parinit,'mu')	% centers not defined
   if (isfield(argout.model,'fam') & strcmp(argout.model.fam,'np'))  
	argout.parinit.mu = 'none';
	argout.parknown.mu = 'none';
   else		
	argout.parinit.mu = 'random';
        argout.parinit.O = 'none';
        argout.parknown.O = 'none';
   end
end
if ~isfield(argout.parinit,'S')	% variance matrices not defined
     if (~isfield(argout.model,'fam') | strcmp(argout.model.fam,'normal'))
          argout.parinit.S = 'identity';
     else
          % no covariance matrix needed
          argout.parinit.S = 'none';
          argout.parknown.S = 'none';
     end
end
if ~isfield(argout.parinit,'O')	% observation matrix not defined
     if (isfield(argout.model,'fam') & strcmp(argout.model.fam,'np'))
          argout.parinit.O = 'random';
	  argout.parinit.mu = 'none';
	  argout.parinit.S = 'none';
	  argout.parknown.mu = 'none';
	  argout.parknown.S = 'none';
     else
          % no observation matrix needed
          argout.parinit.O = 'none';
          argout.parknown.O = 'none';
     end
end

% algo default value
% ------------------

if ~isfield(argout,'algo')
  argout.algo = {'em'};		% default is 'em'
end
if ~iscell(argout.algo)
  argout.algo = {argout.algo};	% algo is a cell: 'em' -> {'em'}
end
nbalgo = length(argout.algo);

% sempar default value
% --------------------

if ~isfield(argout,'sempar')
  argout.sempar = {'mean'};		% default is 'mean'
end
if iscell(argout.sempar) & (length(argout.sempar)==1)
  argout.sempar = argout.sempar{1};
end
if ~iscell(argout.sempar)			% replicate sempar nbalgo times
  tmp = argout.sempar;			% in a cell
  argout.sempar = cell(1,nbalgo);
  for i = 1:nbalgo
    argout.sempar{i} = tmp;
  end
end

% nbxem default value
% -------------------

if ~isfield(argout,'nbxem')				% if nbxem not exist
  if ((strcmp(argout.init,'parameters') & ...		% and random parameters
       (strcmp(argout.parinit.p,'random') | ...
        strcmp(argout.parinit.mu,'random')) | ...
        strcmp(argout.parinit.S,'random var.') | ...
        strcmp(argout.parknown.O,'unknown')) | ...
     (strcmp(argout.init,'partition') & ...		& or random partition
       strcmp(argout.zinit,'random') | ...
       any(isnan(argout.zinit)))) & ...
     (strcmp(argout.zknown,'unknown') | ...		% and no enterely fixed partiton
      any(isnan(argout.zknown)))
    argout.nbxem = 5;				% then nbxem is 5
  else
    argout.nbxem = 1;				% else nbxem is 1
  end
end

% convergence default value
% -------------------------

if ~isfield(argout,'cvg')
  argout.cvg = 'xmlORmaxit';			% default value
end
if iscell(argout.cvg) & (length(argout.cvg)==1)
  argout.cvg = argout.cvg{1};
end
if ~iscell(argout.cvg)			% replicate cvg nbalgo times
  tmp = argout.cvg;				% in a cell
  argout.cvg = cell(1,nbalgo);
  for i = 1:nbalgo
    argout.cvg{i} = tmp;
  end
end

% maxit default value
% -------------------

if ~isfield(argout,'maxit')				% if maxit not exist
  if strcmp(argout.parknown.p,'unknown') | ...		% unknown parameters
     strcmp(argout.parknown.mu,'unknown') | ...
     strcmp(argout.parknown.S,'unknown') | ...
     strcmp(argout.parknown.O,'unknown') | ...
     strcmp(argout.zknown,'unknown') | ...		% and unknown partition
     any(isnan(argout.zknown))
    argout.maxit = 500;				% then maxit is 500
  else
    argout.maxit = 1;				% else maxit is 1
  end
end
if length(argout.maxit)==1				% replicate maxit nbalgo times
  argout.maxit = argout.maxit + zeros(1,nbalgo);
end

% cutsem default value
% --------------------

if ~isfield(argout,'cutsem')				% if cutsem not exist
  argout.cutsem = 10;				% then maxit is 500
end
if length(argout.cutsem)==1				% replicate cutsem nbalgo times
  argout.cutsem = argout.cutsem + zeros(1,nbalgo);
end

% maxfail default value
% ---------------------

if ~isfield(argout,'maxfail')
  argout.maxfail = 5;
end

% saveit default value
% --------------------

if ~isfield(argout,'saveit')
  argout.saveit = 'no';
end

% model default value
% -------------------

if ~isfield(argout,'model')			% all structure not exist
  argout.model = [];
end
if ~isfield(argout.model,'model')		% model not defined
  argout.model.model = 'plI';
end
if ~isfield(argout.model,'fam')		        % family not defined
  argout.model.fam = 'normal';
end
if ~isfield(argout.model,'k')			% number of clusters not defined
  if ~strcmp(argout.zknown,'unknown') & ...	% zknown is at least partially defined 
     any(~isnan(argout.zknown))
    argout.model.k = max(argout.zknown);		% obtain it from zknown
  elseif ~strcmp(argout.parknown.p,'unknown')		% p is known
    argout.model.k = length(argout.parknown.p);		% obtain it from this p
  elseif ~strcmp(argout.parknown.mu,'unknown')		% mu is known
    argout.model.k = length(argout.parknown.mu(:,1));	% obtain it from this mu
  elseif ~strcmp(argout.parknown.S,'unknown')		% S is known
    argout.model.k = length(argout.parknown.S(1,1,:));	% obtain it from this S
  elseif ~isstr(argout.parinit.p)			% p initial is supplied
    argout.model.k = length(argout.parinit.p);		% obtain it from this p
  elseif ~isstr(argout.parinit.mu)			% mu initial is supplied
    argout.model.k = length(argout.parinit.mu(:,1));	% obtain it from this mu
  elseif ~isstr(argout.parinit.S)			% S initial is supplied
    argout.model.k = length(argout.parinit.S(1,1,:));	% obtain it from this S
  elseif ~isstr(argout.parinit.O)			% O initial is supplied
    argout.model.k = length(argout.parinit.O(:,1));	% obtain it from this O
  else
    argout.model.k = ceil(n^0.3);		% default value for k
  end
end
if ~iscell(argout.model.model)
  argout.model.model = {argout.model.model};	% model is a cell: 'plI' -> {'plI'}
end
