function valcrit = normixcrit(x,par,crit,zknown,parknown)

% valcrit = normixcrit(x,par,crit[,zknown,parknown])
% Criteria value in the normal model-based.
% input:
%	x	data [n,d]
%	par	estimated parameters (structure)
%	-.model	model 'plI'...
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	crit	name criterion: 'ml', 'cml', 'bic', 'icl', 'nec'
%	zknown	known labels: 'unknown' or value (NaN values possible) [n] OPTIONAL
%	parknown	known parameters (structure) OPTIONAL
%	-.p	proportions: 'unknown' or anything
%	-.mu	centers: 'unknown' or anything
%	-.S	variance matrices: 'unknown' or anything
% output:
%	bestval	optimal value of the criterion
%	bestmod	optimal associated model (structure)
%	-.k	number of clusters
%	-.model	model 'plI'...
%	allval	all values of (vector of structure [nbk*nbmod])
%	-.k	number of clusters
%	-.model	model
%	-.val	associated criterion

% manage optional arguments
% -------------------------

if nargin <=4		% parknown is not supplied
  parknown.p = 'unknown';
  parknown.mu = 'unknown';
  parknown.S = 'unknown';
end
if nargin == 3		% zknown is not supplied
  zknown = 'unknown';
end

[n,d] = size(x);		% sample size and space dimension
k = length(par.p);		% number of clusters

% compute conditional probabilities and use zknown if present
% -----------------------------------------------------------

[f,sumf,logf,logsumf] = normixpdf(x,par);	% density
t = condprob(logf,logsumf);			% conditional probabilities
if ~strcmp(zknown,'unknown')			% if knowledge on partition may be present
  wherenoNaN = ~isnan(zknown);		% find which part of partition is known
  t(wherenoNaN,:) = lab2bin(zknown(wherenoNaN),k);% modify t at these locations
end
entropy = entro(t);				% entropy of the partition

% compute likelihood or completed likelihood
% ------------------------------------------

switch crit

  case {'ml','bic','nec'}
    valcrit = sum(sum(t .* logf)) + entropy;	% likelihood

  case {'cml','icl'}
    t = lab2bin(labels(t),k);			% map
    valcrit = sum(sum(t .* logf));		% completed likelihood

  otherwise fprintf(1,'unknown criterion'); return

end

% compute bic, icl or nec criterion
% ---------------------------------

switch crit

  case {'bic','icl'}
    model = par.model;			% normal model
    df = normixdf(model,k,d,parknown);		% degrees of freedom
    valcrit = valcrit - df/2 * log(n);		% bic or icl value

  case 'nec'
    if k == 1
      nec = 1;				% nec is 1 for the one cluster case
    else
      model = par.model;			% normal model
      model1.model = model;			% this model
      model1.k = 1;				% with one cluster
      par1 = normixem('x',x,'model',model1,...
             'algo','em','nbxem',1,'maxit',1);	% estimate parameters
      [f,sumf,logf] = normixpdf(x,par1);	% densities at each datum 
      ml1 = sum(logf(:))		;	% associated likelihood
      valcrit = entropy / (valcrit - ml1);	% nec value
  end
end

