function bestpar = normixem(varargin)

% bestpar=normixem(x,zknown,parknown,model,zinit,parinit,algo,nbxem,cvg,maxit,saveit,maxfail,sempar,cutsem)
% Algorithms EM, CEM or SEM in the normal model-based.
% input:
%	x	data [n,d]
%	zknown	known labels: 'unknown' or value (NaN values possible) [n]
%	parknown	known parameters (structure)
%	-.p	proportions: 'unknown' or value [k]
%	-.mu	centers: 'unknown' or value [k,d]
%	-.S	variance matrices: 'unknown' or value [d,d,k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plI' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.fam   family of conditionnal law : 'normal' or 'exp'
%	zinit	initial labels: 'random' or value (NaN values possible) [n]
%	parinit	initial parameters (structure)
%	-.p	proportions: 'equal', 'random' or value [k]
%	-.mu	centers: 'random' or value [k,d]
%	-.S	variance matrices: 'identity', 'rand. var.', 'var. x' or value [d,d,k]
%		NOTE : zinit and parinit cannot be both supplied
%	algo	names of chained algo: 'em', 'cem', 'sem' or any combination of them
%	nbxem	number of trials: 1 if no random init, 5 otherwise or value
%	cvg	convergence mode: 'xml', 'partition', 'maxit',
%                   'xmlORmaxit' or 'partitionORmaxit' or a cell [nbalgo] of them
%	maxit	iteration number: 1 if no random init, 500 otherwise
%                   or a vector [nbalgo] of values
%	saveit	save unused iteration number for the following algo: 'no' or 'yes'
%	maxfail	max. number of failures: 5 or value
%	sempar	estimate sem parameters: 'mean', 'max' or a cell [nbalgo] of them
%	cutsem	number of first iterations to cancel for sem if 'sempar' is 'mean':
%		10 or a cell [nbalgo] of values
% output:
%	bestpar	estimated parameters (vector of structure [nbk*nbmod])
%	-.k	number of clusters
%       -.xml   loglikelihood of bestpar
%	-.model	model
%	-.totalit	total iteration number
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]

% control arguments
% -----------------

args = normixarg(varargin);

nbk = length(args.model.k);		% number of different number of clusters
nbmod = length(args.model.model);	% number of Gaussian models
nbalgo = length(args.algo);		% number of algorithms
[n,d] = size(args.x);			% sample size and space dimension
ikmod = 0;			% number of current model+nb.cl.

% loop on the number of clusters
% ------------------------------

for ik = 1:nbk

  % loop on the model
  % -----------------

  for imod = 1:nbmod

    ikmod = ikmod + 1;

    thismodel = struct('model',args.model.model{imod},...	% current model
                      'k',args.model.k(ik),'fam',args.model.fam);

    message = [num2str(thismodel.k) ' cluster(s) - model ' thismodel.model];
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));
    fprintf(1,'| %s | \n',message);			% display current model
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));

    % loop on the number of trials
    % ----------------------------

    bestxml = -Inf;					% initialize bestxml
    xml = -Inf;						% initialize xml
    bestpar(ikmod).k = thismodel.k;			% save current k
    bestpar(ikmod).model = thismodel.model;		% save current model
    nbfail = 0;						% initialize nbfail
    itrial = 0;						% initialize itrial

    while itrial < args.nbxem

      totalit = 0;					% total used iteration number
      previt = 0;					% unused previous iteration number  

      itrial = itrial + 1;

      if nbfail >= args.maxfail			% maxfail is reached ?
        break					% go to the next model
      end

      fprintf(1,'| xem %d: ',itrial);			% display nb. trials

      % initial parameters
      % ------------------

      switch args.init

        case 'parameters',					% parameters initialization
          parinit = normixpinit(args.parinit,thismodel.k,d,args.x);	% initial parameters
          if isempty(parinit), return, end			% failure is fatal here

        case 'partition'					% partition initialization
          parinit = normixzinit(args.zinit,args.zknown,...
                    thismodel.model,thismodel.k,args.x);		% initial parameters

      end

      % loop on the chained algorithms
      % ------------------------------

      for ialgo = 1:nbalgo

        thisalgo = args.algo{ialgo};			% current algorithm
        thiscvg = args.cvg{ialgo};			% associated convergence
        thismaxit = args.maxit(ialgo)+previt;		% associated thismaxit
        thiscutsem = args.cutsem(ialgo);		% associated cutsem
        thisempar = args.sempar{ialgo};			% associated sempar

        if ~isempty(parinit)
          [parinit,xml,nbit] = normixem1(args.x,parinit,args.parknown,args.zknown,thismodel,...
                               thisalgo,thiscvg,thismaxit,thiscutsem,thisempar);
        end

        if isempty(parinit)			% failure ?
          nbfail = nbfail + 1;		% increase number of failure
          itrial = itrial - 1;		% this trial again
          break				% go to loop of trials
        else
          nbfail = 0;			% no failure for this trial
        end

        totalit = totalit + nbit;		% total used iteration number

        switch args.saveit				% update previt
          case 'yes', previt = thismaxit - nbit;		% unused previous iteration number to save for the next algo
          case 'no',				% no saving
          otherwise fprintf(1,'unknown saveit');		% error
        end

      end % loop on the chained algorithms

      if xml > bestxml			% this run is the best until now ?
        bestxml = xml;			% update xml
        bestpar(ikmod).xml = xml;
        bestpar(ikmod).p = parinit.p;		% and parameters
        bestpar(ikmod).mu = parinit.mu;
        bestpar(ikmod).S = parinit.S;
        bestpar(ikmod).O = parinit.O;
        bestpar(ikmod).totalit = totalit;	% and total used iteration number
      end

      fprintf(1,'\n');

    end % loop on the number of trials

  end % loop on the model

end % loop on the number of clusters�