function [par,xml,nbit] = normixem1(x,parinit,parknown,zknown,model,algo,cvg,maxit,cutsem,sempar)

% [par,xml,nbit] = normixem1(x,parinit,parknown,zknown,model,algo,cvg,maxit,cutsem,sempar)
% One algorithm xem in the normal case: em, cem or sem.
% input:
%	x	data [n,d]
%	parinit	initial parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	parknown	known parameters (structure)
%	-.p	'unknown' or proportions [k]
%	-.mu	'unknown' or centers [k,d]
%	-.S	'unknown' or variance matrices [d,d,k]
%	-.O	'unknown' or obsevation matrix [d,d,k]
%	zknown	'unknown' or known labels (NaN allowed) of the partition [n]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.fam   family of conditionnal law : 'normal', 'exp' or 'np'
%	algo	algorithm 'em', 'cem' or 'sem'
%	cvg	convergence 'xml', 'partition', 'maxit', 'xmlORmaxit' or 'partitionORmaxit'
%	maxit	maximum iterations number
%	cutsem	canceled first iterations number for sem
%	sempar	parameters type from sem
% output:
%	par	estimated parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.O	'unknown' or obsevation matrix [d,d,k]
%	xml	likelihood (em and sem) or classification likelihood (cem)
%	nbit	used iteration number
% note:	par is empty if no convergence

k = length(parinit.p);		% number of clusters
epscvg = 1e-6;			% epsilon for convergence

% modify convergence mode
% -----------------------

switch algo
  case 'sem', cvg = 'maxit';		% sem: only 'maxit' possible
  case 'em'			% em: 'partition' is replaced by 'xml'
    if strcmp(cvg,'partition')
      cvg = 'xml';
    end
end

fprintf(1,'[%s',algo);

% initializations
% ---------------

cvgok = 0;			% convergence flag
nbit = 0;				% iteration number
told = NaN;			% partition
xmlold = -Inf;			% criterion

par = parinit;					% par set to initial par
if ~strcmp(parknown.p,'unknown'), par.p = parknown.p; end	% par set to known par
if ~strcmp(parknown.mu,'unknown'), par.mu = parknown.mu; end
if ~strcmp(parknown.O,'unknown'), par.O = parknown.O; end
if ~strcmp(parknown.S,'unknown'), par.S = parknown.S; end


if strcmp(model.fam,'normal')
     % badly conditioned matrices: exit algorithm
     % ------------------------------------------

     if Sbadcond(par.S)
          par = []; xml = [];
          fprintf(1,'.badly conditioned variance matrices');
	  return
     end
end

sumpar.p = 0 * par.p;	% sum of all parameters for sem
sumpar.mu = 0 * par.mu;
sumpar.S = 0 * par.S;
sumpar.O = 0 * par.O;

% iterations
% ----------

while ~cvgok

  nbit = nbit + 1;		% one more iteration

  if ~mod(nbit,100)
    fprintf(1,'.%d',nbit);	% display every 100 iteration number
  end

  % step E : conditional probabilities
  % ----------------------------------

  [f,sumf,logf,logsumf] = normixpdf(x,par,model);	% density (proportions included)
  t = condprob(logf,logsumf);			% conditional probabilities
  if ~strcmp(zknown,'unknown') & ...		% if knowledge on partition may be present
     ~all(isnan(zknown))
    wherenoNaN = ~isnan(zknown);		% find which part of partition is known
    t(wherenoNaN,:) = lab2bin(zknown(wherenoNaN),k);	% modify t at these locations
  end

  switch algo

    % cem: step C
    % -----------

    case 'cem',
      t = lab2bin(labels(t),k);		% map
      xml = sum(sum(t .* logf)) + entro(t);	% completed likelihood

    % sem: step S
    % -----------

    case 'sem',
      xml = sum(sum(t .* logf)) + entro(t);	% likelihood (caution: before random z)
      t = lab2bin(rndlabel(t),k);

    % em: no specific step
    % --------------------

    case 'em',
      xml = sum(sum(t .* logf)) + entro(t);	% likelihood

    otherwise par = []; xml = []; fprintf(1,'unknown algorithm'); return

  end

  % empty cluster: exit algorithm
  % -----------------------------

  nk = sum(t,1)';
  if emptycla(nk), par = []; xml = []; fprintf(1,'.empty cluster'); return, end

  % step M: update parameters
  % -------------------------

  if strcmp(parknown.p,'unknown') 
    par.p = estimp(nk,model.model);			% proportions if unknown
  end
  if strcmp(parknown.mu,'unknown') & ...
    (strcmp(model.fam,'normal') | strcmp(model.fam,'exp'))
    par.mu = estimu(x,t,nk);			% centers if unknown
  else
    par.mu = 'none';
  end
  if (strcmp(parknown.S,'unknown') & strcmp(model.fam,'normal'))
	% for models with exponential distribution, 
	% parameters can also be estimated from variance
    par.S = estimS(x,par.mu,t,model.model,par.S,nk);	% variance matrices if unknown
  end
  if strcmp(parknown.O,'unknown') & strcmp(model.fam,'np')
    par.O = estimO(x,t,size(par.O,2),nk);
  end

  % badly conditioned matrices: exit algorithm
  % ------------------------------------------

  if strcmp(model.fam,'normal')

     % badly conditioned matrix: exit algorithm 
     % ------------------------------------------ 
	     
     if Sbadcond(par.S) 
         par = []; xml = []; 
	 fprintf(1,'.badly conditioned variance matrix\n'); 
	 return 
     end 
  end

  % save parameters sum or max for sem
  % ----------------------------------

  if strcmp(algo,'sem')
    if strcmp(sempar,'mean') & (nbit > cutsem)	% sempar is mean and enough iterations
      sumpar.p = sumpar.p + par.p;		% proportions
      if ~strcmp(model.fam,'np')
            sumpar.mu = sumpar.mu + par.mu;	% centers
      end
      if strcmp(model.fam,'normal')
            sumpar.S = sumpar.S + par.S;	
	    % variance matrices for normal family
      end
      if strcmp(model.fam,'np')
            sumpar.O = sumpar.O + par.O;	
	    % observation matrix for non parametric family
      end
    elseif strcmp(sempar,'max')		% sempar is max
      if xml > xmlold
        parmax.p = par.p;			% proportions
        parmax.mu = par.mu;			% centers
        parmax.S = par.S;			% variance matrices
        parmax.O = par.O;			% variance matrices
      end
    end
  end

  % convergence ?
  % -------------

  switch cvg

    case 'maxit', cvgok = (nbit >= maxit);	% stop with iteration number
    case 'partition'			% stop with partition stability
      cvgok = all(t == told);			% partition stability ?
      told = t;
    case 'xml',				% stop with criterion stability
      cvgok = (abs(xml-xmlold) <= epscvg);	% xml stability ?
      xmlold = xml;
    case 'xmlORmaxit'			% stop with xml stability or maxit
      cvgok = (nbit >= maxit) | (abs(xml-xmlold) <= epscvg);
      xmlold = xml;
    case 'partitionORmaxit'			% stop with partition stability or maxit
      cvgok = (nbit >= maxit) | all(t == told);
      told = t;
    otherwise par = []; xml = []; fprintf(1,'.unknown convergence'); return

  end

  cvgok = cvgok | (k == 1);			% if one cluster, cvg is on

end % while ~cvgok

% in the sem case, mean of the (nbit-cutsem) previous paremeters if sempar is 'mean'
%                , parameters value of the max xml if sempar is 'max'
% ----------------------------------------------------------------------------------

if strcmp(algo,'sem') & (k > 1)
  switch sempar

    case 'mean'				% mean of the sem parameters
      if cutsem >= nbit			% cutsem must be less than maxit
        par = [];				% if not: exit the algorithm
        xml = [];
        fprintf(1,'.cutsem must be less than maxit');
        return
      end
      par.p = sumpar.p / (nbit - cutsem);	% mean of proportions
      if ~strcmp(model.fam,'np')
            par.mu = sumpar.mu / (nbit - cutsem);	% mean of centers
      end
      if strcmp(model.fam,'normal')
            par.S = sumpar.S / (nbit - cutsem);	
	    % mean of variance matrices for normal family
      end
      if strcmp(model.fam,'np')
            par.O = sumpar.O / (nbit - cutsem);	
	    % mean of observatino matrices for non parametric family
      end

    case 'max'				% parameters of the max xml of sem
      par.p = parmax.p;			% proportions
      par.mu = parmax.mu;			% centers
      par.S = parmax.S;			% variance matrices
      par.O = parmax.O;			% variance matrices
    otherwise par = []; xml = []; fprintf(1,'.unknown sempar'); return

  end

    [f,sumf,logf,logsumf] = normixpdf(x,par,model);		% density (proportions included)
    t = condprob(logf,logsumf);				% conditional probabilities
    xml = sum(sum(t .* logf)) + entro(t);		% criterion computation
    % requires log(proportions) to be in logf
end

% display the iteration number
% ----------------------------

if mod(nbit,100)
  fprintf(1,'.%d',nbit);			% iteration number
end
fprintf(1,']');

