function [f,sumf,logf,logsumf] = normixpdf(x,par,model)

% [f,sumf,logf,logsumf] = normixpdf(x,par,model)
% Normal components and mixture density.(proportions included)
% input:
%	x	data [n,d]
%	par	parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.invS	inverse of S [d,d,k] OPTIONAL
%	-.detS	determinant of S [k] OPTIONAL
%       -.O     observation matrix ('np' family) [k,v]
%	model	model and number of clusters (structure)
%       -.fam   family of conditionnal law : 'normal' or 'exp'
% output:
%	f	components density (proportions included) [n,k]
%	sumf	mixture density [n]
%	logf	log components density (proportions included) [n,k]
%	logsumf log mixture density [n]

% initializations
% ---------------

[n,d] = size(x);		% sample size and dimension space
k = length(par.p);		% number of clusters

if strcmp(model.fam,'normal')
% if invS is not provided: compute it
% -----------------------------------

if ~isfield(par,'invS')
  par.invS = par.S;
  for ik = 1:k
    par.invS(:,:,ik) = inv(par.S(:,:,ik));	% inverse of the kth mat. var.
  end
end

% if detS is not provided: compute it
% -----------------------------------

if ~isfield(par,'detS')
  par.detS = par.p;
  for ik = 1:k
    par.detS(ik) = det(par.S(:,:,ik));		% det. of the kth mat. var.
  end
end

% logf [n,k]
% ----------

logf = zeros(n,k);

for ik = 1:k

  % center the sample: x - muk
  xminusmuk = x - ones(n,1) * par.mu(ik,:);

  % fast calculus of -0.5 * xminusmuk' * inv(S) * xminusmuk
  logf(:,ik) = -0.5 * sum((xminusmuk * par.invS(:,:,ik)) .* xminusmuk,2);

  % log density of the component ik (proportion included)
  logf(:,ik) = log(par.p(ik)) + logf(:,ik) -d/2*log(2*pi) - 0.5*log(par.detS(ik));

end

% logfsort: logf with a permutation of the max of each line with the 1st column
% (useful to limit numerical problems in the calculus of logsumf)
% -----------------------------------------------------------------------------

logfsort = logf;
[tmp,ikmax] = max(logfsort,[],2);	% ikmax : colonne des max pour chaque ligne
ind = sub2ind([n,k],1:n,ikmax')';	% ind : indice des max ds le vecteur logfsort(:)
tmp = logfsort(:,1);		% tmp : 1ere colonne de logfik2
logfsort(:,1) = logfsort(ind);	% 1ere colonne logfik2 remplacee par les max
logfsort(ind) = tmp';		% les places des max remplacees par la 1ere col.

% logsumf [n]
% -----------

logsumf = logfsort(:,1) + log(ones(n,1) + ...
  sum(exp(logfsort(:,2:k) - logfsort(:,1)*ones(1,k-1)),2));

% f [n,k] and sumf [n]
% --------------------

f = exp(logf);
sumf = exp(logsumf);

elseif strcmp(model.fam,'exp')

% for exponential model
% ---------------------

     % logf [n,k]
     % -----------------

     logf = zeros(n,k);


     for ik = 1:k  		
  		logf(:,ik) =  log(par.p(ik)) - log(par.mu(ik)) + (x / (-1 * par.mu(ik)));
                               % - log(par.mu(ik)) * ones(n,1);
     end


    logsumf = log(sum(exp(logf),2));
    f = exp(logf);
    sumf = exp(logsumf);

elseif strcmp(model.fam,'np')
% for non parametric model
% ---------------------

     % logf [n,k]
     % -----------------

    logf = ones(n,1) * log(par.p)' + log(par.O(:,x(:,1)))';


    logsumf = log(sum(exp(logf),2));
    f = exp(logf);
    sumf = exp(logsumf);


    else
       fprintf(1,'unknown family');
       f = zeros(n,k);
    end


