function [x,z] = normixrnd(n,par,model)

% [x,z] = normixrnd(n,par,model)
% A n-sample simulated from the mixture parameters.
% input:
%	n sample size
%	par	parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	model	        model and number of clusters (structure)
%       -.fam           family of conditionnal law : 'normal', 'exp' or 'np'
% output:
%	x	data [n,d]
%	z	labels [n]
%
% Example:
%
% k =
%
%     2     
%
% d = 
%
%     3
%
% p =
%
%    0.4000    0.6000
%
%
% mu =
%
%     0     0     0
%     1    -1     1   
%   
% S=zeros(d,d,k);
%
% for i=1:k       
% S(:,:,i)=i*eye(d,d)    
% end
%
% S(:,:,1) =
%
%     1     0     0
%     0     1     0
%     0     0     1
%
%
% S(:,:,2) =
%
%     2     0     0
%     0     2     0
%     0     0     2     
%
%  par=struct('p',p,'mu',mu,'S',S) 
%
% par =
%
%     p: [0.4000 0.6000]
%    mu: [2x3 double]
%     S: [3x3x2 double]        
%
% model = 
% 
%     fam: 'normal'
%
% [x,z] = normixrnd(6, par)
%
% x =
%
%    0.5689    0.1184    0.9409
%   -0.2556    0.3148   -0.9921
%    0.4662    1.0414    1.2999
%   -0.2959   -0.3510    0.2379
%   -1.4751    0.6232   -1.0078
%   -0.2340    0.7990   -0.7420
%
%
% z =
%
%     1
%     1
%     2
%     1
%     1
%     1               

if ~strcmp(model.fam,'np')
   [k,d] = size(par.mu);	% number of clusters and space dimension
else
   k = size(par.O,1);
   d=1;
end
x = randn(n,d);		% n points in Rd from N(0,I)

% generate n values in dimension 1 from a U(0,1) distribution
c = rand(n,1);		% n real values from U(0,1)

% choose point origin
% -------------------

z = zeros(n,1);		% labels
sump = 0;
for ik = 1:k
  whereisk = (c >= sump);
  z(whereisk) = ik .* ones(sum(whereisk),1);
  sump = sump + par.p(ik);
end
clear c;

% linear transformation of x in order to come from the mixture
% ------------------------------------------------------------

for ik = 1:k

  % all x from the ikth cluster

  whereisk = (z == ik);

   if strcmp(model.fam,'normal')
       x(whereisk,:) = x(whereisk,:) * sqrtm(par.S(:,:,ik)) + ...
            ones(sum(whereisk),1) * par.mu(ik,:);
   elseif strcmp(model.fam,'exp')
       x(whereisk,:) = exprnd(par.mu(ik),sum(whereisk),1);
   elseif strcmp(model.fam,'np')
	v = size(par.O, 2);
	sump = 0;
	c = rand(n, 1);
	for iv = 1:v
		whereisv = (c >= sump);
		x(whereisk & whereisv,:) = ...
		   iv .* ones(sum(whereisk & whereisv),1);
  		sump = sump + par.O(ik, iv);
	end
   end % if
end % for ik
