function [beta1,beta2,beta3] = treebackward(f,x,par)

% [beta1,beta2,beta3]= treebackward(f,x,par)
% Computation of backward probabilities
% i.e.  P(Ti | Si.parent = m), P(Ti | Si = m)
% and P(T(Si.parent)-Ti | Si.parent = m)
% input:
%	f	components density (cell) {N} x [nk,k]
%       x	data (cell of trees) {N}
%	par	parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k] 
%	-.A	transition matrix [k,k]  or cell of transition matrices 
%       -.H     'unknown' or scale parameter [k]
% output:
%	beta1		backward variables (cell) {N} x [n_N,k]
%	beta2		backward variables (cell) {N} x [n_N,k]
%	beta3		backward variables (cell) {N} x [n_N,k]

% initializations
% ---------------

N = size(f,2);		        % sample size 
d = length(par.mu(1,:));
k = length(par.p);		% number of clusters

beta = cell(1,N);

nb_points = length(f{1}(:,1));
nb_scales = log(nb_points + 1) / log(2);

for n = 1:N			% loop on samples

	beta1{n} = zeros(nb_points,k);
               % beta1{n}(1,:) doesn't mean anything
	beta2{n} = zeros(nb_points,k);
	beta3{n} = zeros(nb_points,k);
               % beta3{n}(1,:) == 0 and won't be used

	if iscell(par.A)   % several transition matrices

	% first step
	% ----------

        for pos = 0:2^(nb_scales-1)-1,
	     beta2{n}(2^(nb_scales-1)+pos,:) = f{n}(2^(nb_scales-1)+pos,:);
             beta1{n}(2^(nb_scales-1)+pos,:) = ...
	            beta2{n}(2^(nb_scales-1)+pos,:) * ...
	                  (par.A{2^(nb_scales-1)+pos})';
        end

	% propagation 
	% -----------
        for scale = 2:nb_scales-1,
             for pos = 0:2^(nb_scales-scale)-1,
	           beta2{n}(2^(nb_scales-scale)+pos,:) = ...
                     f{n}(2^(nb_scales-scale)+pos,:) .* ...
                     beta1{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:) .* ...
                     beta1{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:);
	           beta1{n}(2^(nb_scales-scale)+pos,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) * ...
                            (par.A{2^(nb_scales-scale)+pos})';
	           beta3{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) ./ ...
		        beta1{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:);
	           beta3{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) ./ ...
		        beta1{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:);
             end
        end

           
	% termination
	% ----------- 

	beta2{n}(1,:) =  f{n}(1,:) .*  beta1{n}(2,:) .*  beta1{n}(3,:);    
	beta3{n}(2,:) =  beta2{n}(1,:) ./ beta1{n}(2,:);
	beta3{n}(3,:) =  beta2{n}(1,:) ./ beta1{n}(3,:);
        

        else % only one transition matrix

	% first step
	% ----------

        for pos = 0:2^(nb_scales-1)-1,
	     beta2{n}(2^(nb_scales-1)+pos,:) = f{n}(2^(nb_scales-1)+pos,:);
             beta1{n}(2^(nb_scales-1)+pos,:) = ...
	            beta2{n}(2^(nb_scales-1)+pos,:) * (par.A)';
        end

	% propagation 
	% -----------
        for scale = 2:nb_scales,
             for pos = 0:2^(nb_scales-scale)-1,
	           beta2{n}(2^(nb_scales-scale)+pos,:) = ...
                     f{n}(2^(nb_scales-scale)+pos,:) .* ...
                     beta1{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:) .* ...
                     beta1{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:);
	           beta1{n}(2^(nb_scales-scale)+pos,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) * (par.A)';
	           beta3{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) ./ ...
		        beta1{n}(x{n}{2^(nb_scales-scale)+pos}.lc,:);
	           beta3{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:) = ...
	                beta2{n}(2^(nb_scales-scale)+pos,:) ./ ...
		        beta1{n}(x{n}{2^(nb_scales-scale)+pos}.rc,:);
             end
        end     

	end              %    end if
end;
	
	
 
