function [cl] = treecl(tree,htree,par,model)

% [cl] = treecl(tree,htree,par,model)
% computes the completed loglikelihood of par
% input:
%       tree	   observed data (tree cell)
%       htree	   hidden data (tree cell)
%	par	   parameters (structure)
%	-.p	   proportions [k]
%	-.mu	   centers [k,d]
%	-.S	   variance matrices [d,d,k] 
%	-.A	   transition matrix [k,k]  or cell of transition matrices 
%       -.H        'unknown' or scale parameter [k]
%       model      model and number of clusters (structure)
% 	-.model	   model: 'plkCk' or value cell of strings [nbmod]
% 	-.k	   number of clusters: n^0.3 or value [nbk]
%       -.A        type of transition matrix : 'full','utriangular' or 
%                  'udiagonal'
%       -.fam      family of conditionnal law : 'normal' or 'exp'
%
% output:
%       cl         completed loglikelihood
%
% See also CHAINXPDF, TREEVITERBI

% initializations
% ---------------

k = length(par.p);		% number of clusters
x_vec = tree2vec(tree);

% conditional probabilities
% -------------------------


if strcmp(model.fam,'mbf')
     [f,logf] = treexmbfpdf({x_vec},par);        % density
else
     [f,logf] = chainxpdf({x_vec},par,model);      % density
end

nb_points = length(logf{1}(:,1));
nb_scales = log(nb_points + 1) / log(2);

if (~all(f{1}))
     t = [];
     cl = -Inf;
     fprintf(1,' At least one element of sample has conditionnal probability zero');
     return
end	

    
cl = log(par.p(htree{1}.data)) + logf{1}(1,htree{1}.data);

if iscell(par.A)      % several transition matrices

    for scale = nb_scales-1:-1:1,
             for pos = 0:2^(nb_scales-scale)-1,
	         t = 2^(nb_scales-scale)+pos;
                   cl = cl + ...
                        log(par.A{t}(htree{htree{t}.p}.data,htree{t}.data)) ...
                           + logf{1}(t,htree{t}.data);
             end
    end
else           % one transition matrix
    for scale = nb_scales-1:-1:1,
             for pos = 0:2^(nb_scales-scale)-1,
	       t = 2^(nb_scales-scale)+pos;
                   cl = cl + ...
                        log(par.A(htree{htree{t}.p}.data,htree{t}.data)) ...
                           + logf{1}(t,htree{t}.data);
             end
    end

end
	
	
 
