function [beta1,beta2,beta3,xml,ql] = treecondbackward(f,x,par,h_states)

% [beta1,beta2,beta3,xml,ql] = treecondbackward(f,x,par,h_states)
% Computation of conditional backward probabilities
% i.e.  P(Si.parent = m | Ti), P(Si = m | Ti)
% and P(Si = m | Ti) / P(Si.parent = m | Ti)
% input:
%	f	   components density (cell) {N} x [nk,k]
%       x	   data (cell of trees) {N}
%	par	   parameters (structure)
%	-.p	   proportions [k]
%	-.mu	   centers [k,d]
%	-.S	   variance matrices [d,d,k] 
%	-.A	   transition matrix [k,k]  or cell of transition matrices 
%       -.H        'unknown' or scale parameter [k]
%  	h_states   probabilities of hidden states (cell) {N} x [n_N,k]
% output:
%	beta1	   conditional backward variables (cell) {N} x [n_N,k]
%	beta2	   conditional backward variables (cell) {N} x [n_N,k]
%	beta3	   conditional backward variables (cell) {N} x [n_N,k]
%       xml        loglikelihood [N]
%       ql         likelihood quotients [N,n_N]

% initializations
% ---------------

N = size(f,2);		        % sample size 
d = length(par.mu(1,:));
k = length(par.p);		% number of clusters

beta = cell(1,N);
xml = zeros(1,N);
% xml = cell(1,N);

nb_points = length(f{1}(:,1));
nb_scales = log(nb_points + 1) / log(2);

ql = zeros(N,nb_points);

for n = 1:N			% loop on samples

	beta1{n} = zeros(nb_points,k);
               % beta1{n}(1,:) doesn't mean anything
	beta2{n} = zeros(nb_points,k);
	beta3{n} = zeros(nb_points,k);
               % beta3{n}(1,:) == 0 and won't be used

        den = 0;           % P(T(t)) / (P(T(lc(t))) P(T(rc(t))))
        logl = zeros(nb_points,1);    
	% xml{n} = zeros(nb_points,1);

	if iscell(par.A)   % several transition matrices

	% first step
	% ----------

        for pos = 0:2^(nb_scales-1)-1,
	     t = 2^(nb_scales-1)+pos;
	     beta2{n}(t,:) = f{n}(t,:) .* h_states{n}(t,:);
             ql(n,t) = sum(beta2{n}(t,:));
             beta2{n}(t,:) = beta2{n}(t,:) / ql(n,t);
             % logl(t) = 0;
             beta1{n}(t,:) = ((beta2{n}(t,:) ./ h_states{n}(t,:)) ...
			      * (par.A{t})') .* h_states{n}(x{n}{t}.p,:);
             xml(n) = xml(n) + log(ql(n,t));
             % xml{n}(t) = log(ql(n,t));
        end


	% propagation 
	% -----------
        for scale = 2:nb_scales-1,
             for pos = 0:2^(nb_scales-scale)-1,
                   t = 2^(nb_scales-scale)+pos;
	           beta2{n}(t,:) = (f{n}(t,:) .* beta1{n}(x{n}{t}.lc,:) .* ...
                     beta1{n}(x{n}{t}.rc,:)) ./ h_states{n}(t,:);
                   ql(n,t) = sum(beta2{n}(t,:));
                   beta2{n}(t,:) = beta2{n}(t,:) / ql(n,t);
                   logl(t) = log(ql(n,t)) + logl(x{n}{t}.rc) + ...
                                logl(x{n}{t}.lc);
                   % xml{n}(t) = logl(t) + ...
                       % sum(xml{n}(t*2^(scale-1):(t+1)*2^(scale-1)-1));
                   beta1{n}(t,:) = ((beta2{n}(t,:) ./ h_states{n}(t,:)) ...
			      * (par.A{t})') .* h_states{n}(x{n}{t}.p,:);
	           beta3{n}(x{n}{t}.lc,:) =  beta2{n}(t,:) ./ ...
			      beta1{n}(x{n}{t}.lc,:);
	           beta3{n}(x{n}{t}.rc,:) = beta2{n}(t,:) ./ ...
			      beta1{n}(x{n}{t}.rc,:);
             end
        end

        else % only one transition matrix

	% first step
	% ----------

        for pos = 0:2^(nb_scales-1)-1,
	     t = 2^(nb_scales-1)+pos;
	     beta2{n}(t,:) = f{n}(t,:) .* h_states{n}(t,:);
             ql(n,t) = sum(beta2{n}(t,:));
             beta2{n}(t,:) = beta2{n}(t,:) / ql(n,t);
             % logl(t) = 0;
             beta1{n}(t,:) = ((beta2{n}(t,:) ./ h_states{n}(t,:)) ...
			      * (par.A)') .* h_states{n}(x{n}{t}.p,:);
             xml(n) = xml(n) + log(ql(n,t));
             % xml{n}(t) = log(ql(n,t));
        end

	% propagation 
	% -----------
        for scale = 2:nb_scales-1,
             for pos = 0:2^(nb_scales-scale)-1,
                   t = 2^(nb_scales-scale)+pos;
	           beta2{n}(t,:) = (f{n}(t,:) .* beta1{n}(x{n}{t}.lc,:) .* ...
                     beta1{n}(x{n}{t}.rc,:)) ./ h_states{n}(t,:);
                   ql(n,t) = sum(beta2{n}(t,:));
                   beta2{n}(t,:) = beta2{n}(t,:) / ql(n,t);
                   logl(t) = log(ql(n,t)) + logl(x{n}{t}.rc) + ...
                                logl(x{n}{t}.lc);
                   % xml{n}(t) = logl(t) + ...
                       % sum(xml{n}(t*2^(scale-1):(t+1)*2^(scale-1)-1));
                   beta1{n}(t,:) = ((beta2{n}(t,:) ./ h_states{n}(t,:)) ...
			      * (par.A)') .* h_states{n}(x{n}{t}.p,:);
	           beta3{n}(x{n}{t}.lc,:) =  beta2{n}(t,:) ./ ...
			      beta1{n}(x{n}{t}.lc,:);
	           beta3{n}(x{n}{t}.rc,:) = beta2{n}(t,:) ./ ...
			      beta1{n}(x{n}{t}.rc,:);
             end
        end     

	end              %    end if


	% termination
	% ----------- 

	beta2{n}(1,:) = (f{n}(1,:) .* beta1{n}(2,:) .* beta1{n}(3,:)) ...
	                    ./ h_states{n}(1,:);
        ql(n,1) = sum(beta2{n}(1,:));
        beta2{n}(1,:) = beta2{n}(1,:) / ql(n,1);
        logl(1) = log(ql(n,1)) + logl(2) + logl(3);   
	beta3{n}(2,:) =  beta2{n}(1,:) ./ beta1{n}(2,:);
	beta3{n}(3,:) =  beta2{n}(1,:) ./ beta1{n}(3,:);

        xml(n) = logl(1) + xml(n);

        % xml{n}(1) = logl(1) + ...
	   % sum(xml{n}(2^(nb_scales-1):2*2^(nb_scales-1)-1));
end;
	
	
 
