function [ttree,tvec,cml] = treestates(tree,par,model)

% [ttree,tvec,cml] = treestates(tree,par,model)
% Restauration step via MAP algorithm          
% input:
%	tree	data (cell of trees) {N}
%	par    	parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.A   	transition matrix [k,k]
%       -.H     scale parameter [k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix 
%       -.fam   family of conditionnal law : 'normal' or 'exp' or 'mbf'
%       -.hom   homogeneity : 'hom' or 'non_hom'
% output:
%	ttree      hidden states   (cell of trees) {N}
%	tvec       hidden states, vectorised (cell) N x n_N 
%       cml        classification (completed) loglikelihood   
%
% Example : 
%
% p =
%
%    0.4000    0.6000
%   
% A =
% 
%    0.8000    0.2000
%    0.1000    0.9000
%                     
% mu =
%
%    -1    -1    -1
%     1     1     1 
%
% S(:,:,1) =
%
%     1     0     0
%     0     1     0
%     0     0     1
%
%
% S(:,:,2) =
%
%    0.5000         0         0
%         0    0.5000         0
%         0         0    0.5000
%                            
% par=struct('A',A,'p',p,'mu',mu,'S',S);
% 
% model = 
%
%    fam: 'normal'
%    hom: 'hom'
%
% [tree,z] = treexrnd([7], par, model);
%
% [ttree,tvec,cml] = treestates(tree,par,model);
%
%  cml =
%
%  -68.5601
%
%  z_vec = tree2vec(z{1});                       
%
%  any(tvec{1}-z_vec)
%
%  ans =
%
%     0

k = length(par.p);		% number of states
N = length(tree);                  % sample size

for n = 1:N,
    x_vec{n} = tree2vec(tree{n});    % vectorisation of trees
end

  % conditional probabilities
  % ----------------------------------


   if strcmp(model.fam,'mbf')
     [f,logf] = treexmbfpdf(x_vec,par);        % density
   else
     [f,logf] = chainxpdf(x_vec,par,model);      % density
   end

       for n = 1:N
            if (~all(f{n}))
                t = [];
		cml = -Inf;
		fprintf(1,' At least one element of sample has conditional probability zero');
		return
	    end	
	end
    
    % classification step
    % -------------------

    [h_states] = hiddentree(tree,par);
    [beta1,beta2,beta3,xml,ql] = treecondbackward(f,tree,par,h_states);

    [ttree,tvec,cml] = treeviterbilog(logf,par,tree,ql,beta2,h_states);
    cml = cml + xml;    % conditional to joint loglikelihood

    

