function [ttree,tvec,xml] = treeviterbilog(logf,par,x,ql,beta2,h_states)

% [ttree,tvec,xml] = treeviterbilog(logf,par,x,ql,beta2,h_states)
% Restauration step : MAP algorithm
% input:
%	logf	   components log density (cell) {N} x [nk,k]
%	par	   parameters (structure)
%	-.p	   proportions [k]
%	-.mu	   centers [k,d]
%	-.S	   variance matrices [d,d,k] 
%	-.A	   transition matrix [k,k]  or cell of transition matrices 
%       -.H        'unknown' or scale parameter [k]
%       x	   data (cell of trees) {N}
%       ql         likelihood quotients [N,n_N]
%	beta2	   conditional backward variables (cell) {N} x [n_N,k]
%  	h_states   probabilities of hidden states (cell) {N} x [n_N,k]
% output:
%	ttree      hidden states   (cell of trees) {N}
%	tvec       hidden states, vectorised (cell) N x n_N 
%     	xml        classification conditional loglikelihood [N]
%
% See also CHAINXPDF, TREECONDBACKWARD, TREEVITERBI

% initializations
% ---------------

N = length(logf);		        % sample size 
k = length(par.p);		% number of clusters

tvec = cell(1,N);
xml = zeros(1,N);

nb_points = length(logf{1}(:,1));
nb_scales = log(nb_points + 1) / log(2);

for n = 1:N			% loop on samples	
	
	delta1 = zeros(k,nb_points);
	delta2 = zeros(k,nb_points);
	psi = zeros(k,nb_points);

	tvec{n}=zeros(nb_points,1);   % hidden states for nth tree

	% first step
	% ----------

        for pos = 0:2^(nb_scales-1)-1,
	     t = 2^(nb_scales-1)+pos;	
             delta2(:,t) = log(beta2{n}(t,:)');	
        end

	% propagation 
	% -----------

        if iscell(par.A)      % several transition matrices


        for scale = 1:nb_scales-1,
             for pos = 0:2^(nb_scales-scale)-1,
                   t = 2^(nb_scales-scale)+pos;
                   [P,S] = max(((delta2(:,t) - log(h_states{n}(t,:)')) * ...
                            ones(1,k)) + log(par.A{t}) ,[],1);
                   psi(:,t) = S';
	           delta1(:,t) = P' + log(h_states{n}(x{n}{t}.p,:)');
             end
             for pos = 0:2^(nb_scales-scale)-1,
                   t = 2^(nb_scales-scale)+pos;
                   delta2(:,x{n}{t}.p) = -log(ql(n,x{n}{t}.p)) + ...
                   (logf{n}(x{n}{t}.p,:)' - log(h_states{n}(x{n}{t}.p,:)') ...
                             + delta1(:,x{n}{x{n}{t}.p}.lc) + ...
                                delta1(:,x{n}{x{n}{t}.p}.rc));
             end
        end

        else           % one transition matrix


        for scale = 1:nb_scales-1,
             for pos = 0:2^(nb_scales-scale)-1,
                   t = 2^(nb_scales-scale)+pos;
                   [P,S] = max(((delta2(:,t) - log(h_states{n}(t,:)')) * ...
                            ones(1,k)) +log(par.A) ,[],1);
                   psi(:,t) = S';
	           delta1(:,t) = P' + log(h_states{n}(x{n}{t}.p,:)');
             end
             for pos = 0:2^(nb_scales-(scale+1))-1,
                   t = 2^(nb_scales-(scale+1))+pos;
                   delta2(:,t) = -log(ql(n,t)) + (logf{n}(t,:)' ...
		     - log(h_states{n}(t,:)') + delta1(:,x{n}{t}.lc) + ...
                                delta1(:,x{n}{t}.rc));
             end
        end

        end                      % end if

	% termination
	% -----------

        [xml(n),tvec{n}(1,1)] = max(delta2(:,1));

	% path backtracking
	% -----------------

	for t = 2:nb_points
		tvec{n}(t,1) = psi(tvec{n}(x{n}{t}.p,1),t);
	end	
        ttree{n} = vec2tree(tvec{n});
end
	
	
 
