function bestpar = treexem(varargin)

% definitive version (planned) : 
% bestpar=treexem(x,zknown,parknown,model,parinit,algo,nbxem,cvg,maxit,saveit,maxfail,sempar,cutsem)
% Algorithms EM, CEM or SEM in the normal model-based.
% simplified current version :
% bestpar=treexem(x,parknown,model,parinit,algo,nbxem,cvg,maxit,saveit,maxfail)
% Algorithms EM in the normal model-based.
% input:
%	x	data (cell) : N trees {N} 
%	parknown	known parameters (structure)
%	-.p	probability of 1st state: 'unknown' or value [k]
%	-.mu	centers: 'unknown' or value [k,d]
%	-.S	variance matrices: 'unknown' or value [d,d,k]
%	-.A 	transition matrix: 'unknown' or value [k,k] or cell
%       -.H     scale parameter : value [k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix = [-min,max]
%               min > 0 : first non zero sub-diagonal, 
%               max > 0 : last non zero upper-diagonal (min + max > 2)
%       -.fam   family of conditionnal law : 'normal' or 'exp' or 'mbf'
%       -.hom   homogeneity : 'hom' or 'non_hom'
%	parinit	initial parameters (structure)
%	-.p	probability of 1st state: 'equal', 'random' or value [k]
%	-.mu	centers ('normal' or 'exp' families) : 'random' or value [k,d]
%	-.S	variance matrices: 'identity', 'rand. var.', 'var. x' 
%               or value [d,d,k]
%	-.A 	transition matrix: 'random' or value [k,k] or cell
%	algo	names of chained algo: 'em'
%	nbxem	number of trials: 1 if no random init, 5 otherwise or value
%	cvg	convergence mode: 'xml', 'maxit', 'xmlORmaxit' 
%               or a cell [nbalgo] of them
%	maxit	iteration number: 1 if no random init, 500 otherwise
%                   or a vector [nbalgo] of values
%	saveit	save unused iteration number for the following algo:
%               'no' or 'yes'
%	maxfail	max. number of failures: 5 or value
% output:
%	bestpar	estimated parameters (vector of structure [nbk*nbmod])
%	-.k	number of clusters
%       -.xml   loglikelihood of bestpar
%	-.model	model
%	-.totalit	total iteration number
%	-.p	probability of 1st state [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.A 	transition matrix [k,k] or cell of transition matrices
%       -.H     scale parameter : value [k]
%
%  Example :
%
% k =
% 
%     2     
%
% d = 
%
%     3
%
% p =
% 
%    0.1000    0.9000      
%
% mu =
% 
%    -1     2    -1
%     3     0     3      
% S(:,:,1) =
% 
%    0.5000         0         0
%         0    0.5000         0
%         0         0    0.5000
% 
% 
% S(:,:,2) =
% 
%     2     0     0
%     0     2     0
%     0     0     2      
% A =
% 
%    0.7000    0.3000
%    0.2000    0.8000   
% 
%   par=struct('p',p,'mu',mu,'S',S,'A',A)
% 
% par =
% 
%     p: [0.1000 0.9000]
%    mu: [2x3 double]
%     S: [3x3x2 double]
%     A: [2x2 double]     
%
%
% [x,z] = treexrnd([10,14,22,19,7,32], par);
%
%  bestpar = treexem('x',x,'nbxem',40);
%   

% control arguments
% -----------------

args = treexarg(varargin);

nbk = length(args.model.k);	     % number of different number of clusters
nbmod = length(args.model.model);    % number of Gaussian models
nbalgo = length(args.algo);	     % number of algorithms
d = size(args.x{1},2);
ikmod = 0;			% number of current model+nb.cl.
N = size(args.x,2);                  % number of trees

for n = 1:N,
    args.x_vec{n} = tree2vec(args.x{n});    % vectorisation of trees
end


% loop on the number of clusters
% ------------------------------

for ik = 1:nbk

  % loop on the model
  % -----------------
  
  for imod = 1:nbmod

    ikmod = ikmod + 1;

    thismodel = struct('model',args.model.model{imod},...
		       'k',args.model.k(ik),'fam',args.model.fam,...
		       'A',args.model.A{ik},...
		       'hom',args.model.hom);	% current model

    message = [num2str(thismodel.k) ' state(s) - model ' thismodel.model];
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));
    fprintf(1,'| %s | \n',message);			% display current model
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));

    % loop on the number of trials
    % ----------------------------

    bestxml = -Inf;				% initialize bestxml
    xml = -Inf;					% initialize xml
    bestpar(ikmod).k = thismodel.k;		% save current k
    bestpar(ikmod).model = thismodel.model;	% save current model
    nbfail = 0;					% initialize nbfail
    itrial = 0;					% initialize itrial

    while itrial < args.nbxem

      totalit = 0;				% total used iteration number
      previt = 0;				% unused previous iteration number  

      itrial = itrial + 1;

      if nbfail >= args.maxfail			% maxfail is reached ?
        break					% go to the next model
      end

      fprintf(1,'| xem %d: ',itrial);			% display nb. trials

      % initial parameters
      % ------------------

      
      parinit = treexpinit(args.parinit, thismodel.k, d,...
			   args.x_vec, thismodel.A, ...
			   thismodel.fam,thismodel.hom);

	% initial parameters
      if isempty(parinit), return, end		 % failure is fatal here
      
      % loop on the chained algorithms
      % ------------------------------

      for ialgo = 1:nbalgo

        thisalgo = args.algo{ialgo};			% current algorithm
        thiscvg = args.cvg{ialgo};		     % associated convergence
        thismaxit = args.maxit(ialgo)+previt;		% associated thismaxit
        %thiscutsem = args.cutsem(ialgo);		% associated cutsem
	%thisempar = args.sempar{ialgo};		% associated sempar
      
      	if ~isempty(parinit)
     	[parinit,xml,nbit] = treexem1(args.x,args.x_vec,parinit,...
				      args.parknown,thismodel,thisalgo,...
				      thiscvg,thismaxit);
	end

        if isempty(parinit)			% failure ?
          nbfail = nbfail + 1;		% increase number of failure
          itrial = itrial - 1;		% this trial again
          break				% go to loop of trials
        else
          nbfail = 0;			% no failure for this trial
        end

        totalit = totalit + nbit;		% total used iteration number

        switch args.saveit				% update previt
          case 'yes', previt = thismaxit - nbit;		
              % unused previous iteration number to save for the next algo
          case 'no',				% no saving
          otherwise fprintf(1,'unknown saveit');		% error
        end

      end % loop on the chained algorithms

      if (~isempty(xml) & (xml > bestxml))  	% this run is the best until now ?
        bestxml = xml;			% update xml
        bestpar(ikmod).xml = xml;
        bestpar(ikmod).p = parinit.p;		% and parameters
        bestpar(ikmod).mu = parinit.mu;
        bestpar(ikmod).S = parinit.S;
        if strcmp(thismodel.fam,'mbf')
              bestpar(ikmod).H = parinit.H;
        end
	bestpar(ikmod).A = parinit.A;
        bestpar(ikmod).totalit = totalit;	% and total used iteration number
      end

      fprintf(1,'\n');

    end % loop on the number of trials

  end % loop on the model

end % loop on the number of clusters
