function [par,xml,nbit] = treexem1(x,x_vec,parinit,parknown,model,algo,cvg,maxit)

% [par,xml,nbit] = treexem1(x,x_vec,parinit,parknown,model,algo,cvg,maxit)
% One algorithm xem in the normal case: em, cem or sem.
% input:
%	x	data (cell of trees) {N}
%       x_vec   data (cell of vectorised trees) {N}
%	parinit	initial parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.A   	transition matrix [k,k]
%       -.H     scale parameter [k]
%	parknown	known parameters (structure)
%	-.p	'unknown' or proportions [k]
%	-.mu	'unknown' or centers/means [k,d]
%	-.S	'unknown' or variance matrices [d,d,k]
%	-.A   	'unknown' or transition matrix [k,k]
%       -.H     'unknown' or scale parameter [k]
%	model	model and number of clusters (structure)
%	-.model	model: 'plkCk' or value cell of strings [nbmod]
%	-.k	number of clusters: n^0.3 or value [nbk]
%       -.A     type of transition matrix 
%       -.fam   family of conditionnal law : 'normal' or 'exp' or 'mbf'
%       -.hom   homogeneity : 'hom' or 'non_hom'
%	algo	algorithm 'em', 'cem', 'gibbs', 
%               'sem' or any combination or them (cell)
%	cvg	convergence 'xml', 'maxit' or 'xmlORmaxit'
%	maxit	maximum iterations number
% output:
%	par	estimated parameters (structure)
%	-.p	proportions [k]
%	-.mu	centers [k,d]
%	-.S	variance matrices [d,d,k]
%	-.A 	transition matrix [k,k]
%       -.H     scale parameter [k]
%	xml	loglikelihood (em and sem) or 
%               classification  loglikelihood (cem)
%	nbit	used iteration number
% note:	par is empty if no convergence
%
% Example : 
%
% x=cell(1,1);
% x =  
%     {[]}  
%
% x{1}=[-1.2,3;1.3,5.4;-01,-0.5];     
% x = 
%     [3x2 double]  
%
%  p=[0.368938, 0.243353, 0.387709]; 
% p =  
%     0.3689    0.2434    0.3877     
%
%    S=zeros(2,2,3);
% 
% S(:,:,1) =
%
%      0     0
%      0     0
%
% 
% S(:,:,2) =
%
%      0     0
%      0     0
%
%
% S(:,:,3) =
%
%      0     0
%      0     0
%
% S(:,:,1)=[0.767118,0.204696;0.204696 , 0.954253];
%
% S(:,:,2)=[0.678661,0.551735;0.551735,0.689453];
%
% S(:,:,3)=[0.647363, 0.219151;0.219151, 0.229258]
% 
% S(:,:,1) =
% 
%     0.7671    0.2047
%     0.2047    0.9543
%
%
% S(:,:,2) =
% 
%     0.6787    0.5517
%     0.5517    0.6895
%
%
% S(:,:,3) =
% 
%     0.6474    0.2192
%     0.2192    0.2293
%
% mu=[0.846233,0.663049;0.222531,0.823503;0.115978,0.787944];
%
% mu =
%
%     0.8462    0.6630
%     0.2225    0.8235
%     0.1160    0.7879
%
%  A=[0.524478,0.183443,0.292079;
%     0.205076,0.400017,0.394908;
%     0.148128,0.0618789,0.789993];
%
% A =
%
%     0.5245    0.1834    0.2921
%     0.2051    0.4000    0.3949
%     0.1481    0.0619    0.7900    
%
%  par=struct('p',{p},'mu',{mu},'A',{A},'S',{S})
% 
% par =
% 
%      p: [0.3689 0.2434 0.3877]
%     mu: [3x2 double]
%      A: [3x3 double]
%      S: [2x2x3 double]                   

k = length(parinit.p);		% number of clusters
N = length(x);                  % sample size
epscvg = 1e-6;			% epsilon for convergence

% modify convergence mode
% -----------------------

switch algo
  case 'sem', cvg = 'maxit';		% sem: only 'maxit' possible
  case 'em'			% em: 'partition' is replaced by 'xml'
    if strcmp(cvg,'partition')
      cvg = 'xml';
    end
  case 'gibbs'
    if strcmp(cvg,'partition')
      cvg = 'xml';
    end
end

fprintf(1,'[%s',algo);

% initializations
% ---------------

cvgok = 0;			% convergence flag
nbit = 0;			% iteration number
told = NaN;			% partition
xmlold = -Inf;			% criterion

par = parinit;			% par set to initial par
if ~strcmp(parknown.p,'unknown'),
  par.p = parknown.p; end % par set to known par
if ~strcmp(parknown.mu,'unknown'), par.mu = parknown.mu; end
if ~strcmp(parknown.S,'unknown'), par.S = parknown.S; end
if ~strcmp(parknown.A,'unknown'), par.A = parknown.A; end
if ~strcmp(parknown.H,'unknown'), par.H = parknown.H; end

if strcmp(model.fam,'normal')
     % badly conditioned matrix: exit algorithm
     % ------------------------------------------

     if Sbadcond(par.S)
         par = []; xml = [];
	 fprintf(1,'.badly conditioned variance matrix');
     return
     end
end

if strcmp(algo,'gibbs')
     zold = cell(N);
     for n = 1:N
         zold{n} = zeros(size(x{n},1),k);
     end
     [f,logf] = treexpdf(x,par,model.model);
     zold = treegibbsinit(par,f);
end

sumpar.p = 0 * par.p;	 % sum of all parameters for sem
sumpar.mu = 0 * par.mu;
if ~strcmp(model.fam,'exp')
     sumpar.S = 0 * par.S;
end
if strcmp(model.hom,'hom')
     sumpar.A = 0 * par.A;
end
if strcmp(model.fam,'mbf')
     sumpar.H = 0 * par.H;
end

% iterations
% ----------

while ~cvgok

  nbit = nbit + 1;		% one more iteration

  if ~mod(nbit,100)
    fprintf(1,'.%d',nbit);	% display every 100 iteration number
  end

  % step E : conditional probabilities
  % ----------------------------------


  if strcmp(model.fam,'mbf')
     [f,logf] = treexmbfpdf(x_vec,par);        % density
  else
     [f,logf] = chainxpdf(x_vec,par,model);      % density
  end
			
       for n = 1:N
            if (~all(f{n}))
                par = [];
		xml = -Inf;
		I = find(f{n} == 0);
		I
%     f{n}(I,:)
% logf{n}(I,:)
		fprintf(1,' At least one element of sample has conditionnal probability zero');
		return
	    end	
	end

    switch algo   
    
    % cem: step C 
    % ----------- 
 
    case 'cem', 

        if all(par.A)
           % viterbilog should be more accurate than viterbi
           % but requires A > 0
           [t,ml] = treeviterbilog(logf,par);     % map
	else 
	   [t,ml] = treeviterbi(f,par);
        end;

        [z_zsuiv,z] = treelab2bin(t,k);   % binary labels

    % sem: step S 
    % ----------- 
 
    case 'sem', 
      
      [alpha,ml] = treeforward(f,par);        % forward algorithm   
	                                      % loglikelihood of par
                                              % (caution: before random z) 

      [beta1,beta2,beta3] = treebackward(f,x,par);    % backward algorithm

      [z_zsuiv,z] = treelab2bin(treerndlabel(beta,f,exp(ml),par),k);

    % gibbs: step S 
    % ------------- 
 
    case 'gibbs', 

      	
      [alpha,ml] = treeforward(f,par);        % forward algorithm
                                              % useful to calculate ml
       ml = treeml(probx);                           % loglikelihood of par
                                              % (caution: before random z) 

	[z_zsuiv,z] = treelab2bin(treegibbslabel(par,f,zold),k);
        for n = 1:N
	     zold{n} = bin2lab(z{n});
        end
 
    % em: no specific step 
    % -------------------- 
 
    case 'em', 

      [h_states] = hiddentree(x,par);
      [beta1,beta2,beta3,ml,ql] = treecondbackward(f,x,par,h_states);
      alpha = treecondforward(x,par,beta3,h_states);


      % if ~all(ml)
      if ~all(isfinite(ml))
             par = []; xml = [];   
             fprintf(1,'\n at least one tree has probability density 0\n');
             return;          
      end 

      [z_zsuiv,z] = treecondprobc(alpha,beta2,beta3,x,h_states,par);
                   % conditional probabilities
   
      for n = 1:N
	    if ~all(isfinite(z{n}))
                par = []; xml = [];   
                fprintf(1,'can''t estimate parameters\n');
                return;
            end
      end
 
    otherwise par = []; xml = []; fprintf(1,'unknown algorithm'); return 
 
  end 
		   
  xml = ml;
    
  % step M: update parameters 
  % ------------------------- 
 
  if strcmp(parknown.mu,'unknown') 
    [totalz,par.mu] = estimuc(x_vec,z);		% centers if unknown 
        if isempty(par.mu)
             par = []; xml = [];   
             fprintf(1,'can''t estimate centers\n');
             return 
        end
   else
         par.mu = parknown.mu;
         d = size(x_vec{1},2);	% dimension of data

         totalmu = zeros(k,d,N);	
         totalz = zeros(k,N);	
         % totalmu ./  totalz = estimated centers for each element in sample

         for n = 1:N			% loop on sample
                  nk = size(z{n},1);	% length of nth sequence
                  totalz(:,n) = (z{n})' * ones(nk,1);
         end;

         totalz = sum(totalz,2);

         if ~all(totalz)
                  fprintf(1,'\n at least one state unused; can''t estimate centers\n');
                 par.mu = [];
		 par = []; xml = [];  
	  end
     end
		 
     if (strcmp(parknown.S,'unknown') & strcmp(model.fam,'normal'))
  	par.S = estimSc(x_vec,par.mu,z,model.model,totalz,par.S);
	% variance matrices if unknown       
	if isempty(par.S)
             par = []; xml = [];   
             fprintf(1,'can''t estimate covariance matrices\n');
             return 
        end
       	else if (strcmp(parknown.S,'unknown') & strcmp(model.fam,'mbf'))
  	     [par.S,par.H] = estimSHt(x_vec,z);
	     % variance and scale parameter if unknown       
	     if (isempty(par.S) | isempty(par.H))
                 par = []; xml = [];   
                 fprintf(1,'can''t estimate covariance matrices or scale parameter\n');
                 return 
             end
        end
    end


    if strcmp(parknown.A,'unknown') 
       par.A = estimAt(z_zsuiv,par.A);
       % transition matrix if unknown 
       if isempty(par.A)
             par = []; xml = [];   
             fprintf(1,'can''t estimate transition matrix\n');
             return 
       end
    end   

  if strcmp(parknown.p,'unknown') 
       par.p = estimpc(z,model.model,par.A);
       % proportions if unknown        
       if isempty(par.p)
             par = []; xml = [];   
             fprintf(1,'can''t estimate proportions\n');
             return   
       end
  end   

  if strcmp(model.fam,'normal')

     % badly conditioned matrix: exit algorithm 
     % ------------------------------------------ 
	     
     if Sbadcond(par.S) 
         par = []; xml = []; 
	 fprintf(1,'.badly conditioned variance matrix\n'); 
	 return 
     end 
  end

  % save parameters sum or max for sem 
  % ---------------------------------- 
 
  if strcmp(algo,'sem') 
	if strcmp(sempar,'mean') & (nbit > cutsem)	
		% sempar is mean and enough iterations       

		sumpar.p = sumpar.p + par.p;		% proportions      
		sumpar.mu = sumpar.mu + par.mu;		% centers  
		if strcmp(model.fam,'normal')
                     sumpar.S = sumpar.S + par.S;	
		     % variance matrices for normal family
		end
                if strcmp(model.fam,'mbf')
                     sumpar.S = sumpar.S + par.S;
                     sumpar.H = sumpar.H + par.H;	
		end
                if strcmp(model.hom,'hom')
		     sumpar.A = sumpar.A + par.A;
                end
	
	elseif strcmp(sempar,'max') 	 % sempar is max  
	      if xml > xmlold 
        	parmax.p = par.p;			% proportions 
	        parmax.mu = par.mu;			% centers 
                if ~strcmp(model.fam,'exp')
		    parmax.S = par.S;			% variance matrices 
                end			% centers 
                if strcmp(model.fam,'mbf')
		     parmax.H = par.H;			% scale parameters
                end
 		parmax.A = par.A;                       % transition matrix
      		end 
	end 
  end 
 
  % convergence ? 
  % ------------- 
 
  switch cvg 
 
    case 'maxit', cvgok = (nbit >= maxit);	% stop with iteration number 
    case 'xml',				% stop with criterion stability 
      cvgok = (abs(xml-xmlold) <= epscvg) ;	% xml stability ? 
      xmlold = xml; 
    case 'xmlORmaxit'			% stop with xml stability or maxit 
      cvgok = (nbit >= maxit) | (abs(xml-xmlold) <= epscvg); 
      xmlold = xml;     
    otherwise par = []; xml = []; fprintf(1,'.unknown convergence'); return 
 
  end 
 
  cvgok = cvgok | (k == 1);			% if one cluster, cvg is on 
 
end % while ~cvgok 

%  in the sem case, mean of the (nbit-cutsem) previous paremeters 
%  if sempar is 'mean' 
%                , parameters value of the max xml if sempar is 'max'  
%----------------------------------------------------------------------------------   
if strcmp(algo,'sem') & (k > 1) 
  switch sempar 
 
    case 'mean'				% mean of the sem parameters 
      if cutsem >= nbit			% cutsem must be less than maxit 
        par = [];				% if not: exit the algorithm 
        xml = []; 
        fprintf(1,'.cutsem must be less than maxit'); 
        return 
      end 
	par.p = sumpar.p / (nbit - cutsem);	% mean of proportions 
      	par.mu = sumpar.mu / (nbit - cutsem);	% mean of centers 
	if strcmp(model.fam,'normal')
            par.S = sumpar.S / (nbit - cutsem);	
	    % mean of variance matrices for normal family
	end
        if strcmp(model.fam,'mbf')
            par.S = sumpar.S / (nbit - cutsem);
            par.H = sumpar.H / (nbit - cutsem);
	end
        if strcmp(model.hom,'hom')
	       par.A = sumpar.A / (nbit - cutsem);   	
               % mean of transition matrices
        end
 
    case 'max'				% parameters of the max xml of sem 
	par.p = parmax.p;			% proportions 
      	par.mu = parmax.mu;			% centers 
        if ~strcmp(model.fam,'exp')
      	     par.S = parmax.S;			% variance matrices 
        end
	par.A = parmax.A;			% transition matrix
        if strcmp(model.fam,'mbf')
	     par.H = parmax.H;			% scale parameter
        end
    otherwise par = []; xml = []; fprintf(1,'.unknown sempar'); return 
 
  end 
 
	[condalpha,probx] = condforward(f,par);     % forward algorithm
	ml = treeml(probx);    	
	xml = sum(ml);				    % criterion computation  
end 
 
% display the iteration number 
% ---------------------------- 
 
if mod(nbit,100) 
  fprintf(1,'.%d',nbit);			% iteration number 
end 
fprintf(1,']'); 
 
