function [tree,z] = treexrnd(seq_scale,par,model)

% [tree,z] = treexrnd(seq_scale,par,model)
% A n-sample simulated from the mixture parameters.
% input:
%	seq_scale 	depth of each tree [N]
%	par		parameters (structure)
%	-.p             law of first state [k]
%	-.mu            centers [k,d]
%	-.S             variance matrices [d,d,k]
%	-.A             transition matrix [k,k]
%	-.H             scale parameter [k]
%	model           model and number of clusters (structure)
%	-.model	        model: 'plkCk' or value cell of strings [nbmod]
%	-.k	        number of clusters: n^0.3 or value [nbk]
%	-.A             type of transition matrix : 'full','utriangular' or 
%                       'udiagonal'
%	-.fam           family of conditionnal law : 'normal', 'exp' or 'mbf'
% output:
%	tree            data (cell of trees) {N}
%	z               states (cell of trees) {N} 
%
% Example:
%
% k =
%
%     2     
%
% d = 
%
%     1
%
% p =
%
%    0.2000    0.8000
%
%
% mu =
%
%     0
%     0
%   
% S=zeros(d,d,k);
%
% for i=1:k       
% S(:,:,i)=i*eye(d,d)    
% end
%
% S(:,:,1) =
%
%     0.5
%
%
% S(:,:,2) =
%
%     2
%
% A=[0.3, 0.7; 0.9, 0.1];
%
% A =
% 
%     0.3000    0.7000
%     0.9000    0.1000  
%                               
%  par=struct('p',p,'mu',mu,'S',S,'A',A) 
%
% par =
%
%      p: [0.2000 0.8000]
%     mu: [2x1 double]
%      S: [1x1x2 double]
%      A: [2x2 double]
%                          
%  model.fam = 'normal';
%
% [tree,z] = treexrnd([6], par, model)
%
%
% tree = 
%
%     {63x1 cell}
%
%
% z = 
%
%     {63x1 cell}
%
                  
[k,d] = size(par.mu);	% number of clusters and space dimension
N = length(seq_scale);	% size of sample

for n = 1:N		% loop on sample

	sk = seq_scale(n);		% max scale of nth sequence
	tree{n} = cell(2^sk-1,1);

        if ~iscell(par.A)
             A = par.A;     
             par.A = cell(2^sk-1);	   
             for t = 2:2^sk-1,
             	     par.A{t} = A;
             end;
        end % end if

	% generate 2^sk-1 values in dimension 1 from a U(0,1) distribution
	c = rand(2^sk-1,1);		% sk real values from U(0,1)

	% choose point origin
	% -------------------

	z{n} = cell(2^sk-1,1);		% hidden states
	sump = 0;
	ik = 0;
	while ((c(1,1) >=  sump) & (ik < k))
		ik = ik + 1;
		sump = sump + par.p(ik);
		z{n}{1}.data = ik;
	end;
	z{n}{1}.p  = 0;	
	z{n}{1}.lc = 2;
	z{n}{1}.rc = 3;

	tree{n}{1}.p  = 0;	
	tree{n}{1}.lc = 2;
	tree{n}{1}.rc = 3;

	switch  model.fam
	case 'normal'
		tree{n}{1}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{1}.data)) ...
                       + par.mu(z{n}{1}.data,:);
        case 'exp'
		tree{n}{1}.data = ...
	               exprnd(par.mu(z{n}{1}.data,:),1,1);
        case 'mbf'
                tree{n}{1}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{1}.data) * ...
			   2^(2*sk*par.H(z{n}{1}.data)));

        otherwise tree = {}; z = {}; ...
                  fprintf(1,'unknown parametric family'); return 
        end

	% Markov tree
	% ------------

	for scale = sk-1:-1:2,
	    for pos = 0:2^(sk-scale)-1,
		sump = 0;
		ik = 0;
                z{n}{2^(sk-scale)+pos}.p     =  floor((2^(sk-scale)+pos) / 2);
                z{n}{2^(sk-scale)+pos}.lc    =  2 * (2^(sk-scale)+pos);
                z{n}{2^(sk-scale)+pos}.rc    =  2 * (2^(sk-scale)+pos)+1;
                tree{n}{2^(sk-scale)+pos}.p  =  floor((2^(sk-scale)+pos) / 2);
                tree{n}{2^(sk-scale)+pos}.lc =  2 * (2^(sk-scale)+pos);
                tree{n}{2^(sk-scale)+pos}.rc =  2 * (2^(sk-scale)+pos)+1;

		while ((c(2^(sk-scale)+pos,1) >=  sump) & (ik < k))
			ik = ik + 1;
			sump = sump + ...
		             par.A{2^(sk-scale)+pos}...
		                (z{n}{z{n}{2^(sk-scale)+pos}.p}.data,ik);
			z{n}{2^(sk-scale)+pos}.data = ik;
		end;


        	switch model.fam 
		case 'normal'
                     tree{n}{2^(sk-scale)+pos}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{2^(sk-scale)+pos}.data)) ...
                       + par.mu(z{n}{2^(sk-scale)+pos}.data,:);
                case 'exp'
		    tree{n}{2^(sk-scale)+pos}.data = ...
	                       exprnd(par.mu(z{n}{2^(sk-scale)+pos}.data,:));
                case 'mbf'
                     tree{n}{2^(sk-scale)+pos}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{2^(sk-scale)+pos}.data) * ...
			   2^(2*scale*par.H(z{n}{2^(sk-scale)+pos}.data)));

%  + par.mu(z{n}{2^(sk-scale)+pos}.data,:);
%  data are centered here

                otherwise tree = {}; z = {}; ...
                      fprintf(1,'unknown parametric family'); return 

                end
            end
	end;

	for pos = 0:2^(sk-1)-1,
		sump = 0;
		ik = 0;
                z{n}{2^(sk-1)+pos}.p     =  floor((2^(sk-1)+pos) / 2);
                z{n}{2^(sk-1)+pos}.lc    =  0;
                z{n}{2^(sk-1)+pos}.rc    =  0;
                tree{n}{2^(sk-1)+pos}.p  =  floor((2^(sk-1)+pos) / 2);
                tree{n}{2^(sk-1)+pos}.lc =  0;
                tree{n}{2^(sk-1)+pos}.rc = 0;

		while ((c(2^(sk-1)+pos,1) >=  sump) & (ik < k))
			ik = ik + 1;
			sump = sump + ...
		             par.A{2^(sk-1)+pos}...
		                 (z{n}{z{n}{2^(sk-1)+pos}.p}.data,ik);
			z{n}{2^(sk-1)+pos}.data = ik;
		end;

        	switch model.fam 
		case 'normal'
                     tree{n}{2^(sk-1)+pos}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{2^(sk-1)+pos}.data)) ...
                       + par.mu(z{n}{2^(sk-1)+pos}.data,:);
                case 'exp'
		    tree{n}{2^(sk-1)+pos}.data = ...
	                       exprnd(par.mu(z{n}{2^(sk-1)+pos}.data,:));
                case 'mbf'
                     tree{n}{2^(sk-1)+pos}.data = randn(1,d)  * ...
                     sqrtm(par.S(:,:,z{n}{2^(sk-1)+pos}.data) * ...
			   2^(2*1*par.H(z{n}{2^(sk-1)+pos}.data)));

%  + par.mu(z{n}{2^(sk-1)+pos}.data,:);
%  data are centered here

                otherwise tree = {}; z = {}; ...
                      fprintf(1,'unknown parametric family'); return 


                end
            end

	clear c;
end
