function bestpar = valxem(varargin)
 
% bestpar=valxem(x,miss,parknown,model,parinit,nbxem,cvg,maxit,saveit,maxfail)
% Algorithms EM in the normal model-based with data totally missing at random
%
% input:
%	x		data: value {N} x [T_N,d] (cell)
%	miss		index of missing values: {M} [m_N] (cell)
%	parknown	known parameters (structure)
%	-.p		proportions: 'unknown' or value [k]
%	-.mu		centers: 'unknown' or value [k,d]
%	-.S		variance matrices: 'unknown' or value [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A 		transition matrix: 'unknown' or value [k,k] 
%	model		model and number of clusters (structure)
%	-.model		model: 'plkCk' or value cell of strings [nbmod]
%	-.k		number of clusters: n^0.3 or value [nbk]
%	-.A		type of transition matrix = [-min,max]
% 			min > 0 : first non zero sub-diagonal, 
% 			max > 0 : last non zero upper-diagonal (min + max > 2)
%	-.fam		family of conditionnal law : 'normal', 'exp' or 'np'
%	parinit		initial parameters (structure)
%	-.p		proportions: 'stationnary', 'random' or value [k]
%	-.mu		centers ('normal' or 'exp' families) : 'random' or value [k,d]
%	-.S		variance matrices: 'identity', 'rand. var.', 'var. x' or value [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A 		transition matrix: 'random' or value [k,k] 
%	nbxem		number of trials: 1 if no random init, 5 otherwise or value
%	cvg		convergence mode: 'xml', 'maxit', 'xmlORmaxit' or a cell [nbalgo] of them
%	maxit		iteration number: 1 if no random init, 500 otherwise
%			or a vector [nbalgo] of values
%	saveit		save unused iteration number for the following algo: 'no' or 'yes'
%	maxfail		max. number of failures: 5 or value
% output:
%	bestpar		estimated parameters (vector of structure [nbk*nbmod])
%	-.k		number of clusters
%	-.xml		loglikelihood of bestpar
%	-.model		model
%	-.totalit	total iteration number
%	-.p		proportions [k]
%	-.mu		centers [k,d]
%	-.S		variance matrices [d,d,k]
%       -.O             observation matrices [k,v]
%	-.A 		transition matrix [k,k] 
%
%  Example :
%
% k =
% 
%     2     
%
% d = 
%
%     3
%
% p =
% 
%    0.1000    0.9000      
%
% mu =
% 
%    -1     2    -1
%     3     0     3      
% S(:,:,1) =
% 
%    0.5000         0         0
%         0    0.5000         0
%         0         0    0.5000
% 
% 
% S(:,:,2) =
% 
%     2     0     0
%     0     2     0
%     0     0     2      
% A =
% 
%    0.7000    0.3000
%    0.2000    0.8000   
% 
%   par=struct('p',p,'mu',mu,'S',S,'A',A)
% 
% par =
% 
%     p: [0.1000 0.9000]
%    mu: [2x3 double]
%     S: [3x3x2 double]
%     A: [2x2 double]     
%
%
% [x,z] = chainxrnd([32], par);
%
%  bestpar = valxem('x',x,'miss',{[2 5 11 14 17 23 28 32]},'nbxem',40);
%   

% control arguments
% -----------------
									  
args = valxarg(varargin);

nbk = length(args.model.k);		% number of different number of clusters
nbmod = length(args.model.model);	% number of Gaussian models
d = size(args.x{1},2);
ikmod = 0;			% number of current model+nb.cl.

% loop on the number of clusters
% ------------------------------

for ik = 1:nbk

  % loop on the model
  % -----------------
  
  for imod = 1:nbmod

    ikmod = ikmod + 1;

    thismodel = struct('model',args.model.model{imod},...
		       'k',args.model.k(ik),'fam',args.model.fam,...
		       'A',args.model.A{ik});	% current model

    message = [num2str(thismodel.k) ' state(s) - model ' thismodel.model];
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));
    fprintf(1,'| %s | \n',message);			% display current model
    fprintf(1,'|%s| \n',repmat('-',1,length(message)+2));

    % loop on the number of trials
    % ----------------------------

    bestxml = -Inf;				% initialize bestxml
    xml = -Inf;					% initialize xml
    bestpar(ikmod).k = thismodel.k;		% save current k
    bestpar(ikmod).model = thismodel.model;	% save current model
    nbfail = 0;					% initialize nbfail
    itrial = 0;					% initialize itrial

    while itrial < args.nbxem

      totalit = 0;				% total used iteration number
      previt = 0;				% unused previous iteration number  

      itrial = itrial + 1;

      if nbfail >= args.maxfail			% maxfail is reached ?
        break					% go to the next model
      end

      fprintf(1,'| xem %d: ',itrial);			% display nb. trials

      % initial parameters
      % ------------------

      
      [parinit,obs] = valxpinit(args.parinit,...
			  thismodel.k,...
			  d,...
			  args.x, ...
			  args.miss, ...
			  thismodel.A);
        % only observed values are used in parinit (initial parameters)

      if isempty(parinit), return, end			% failure is fatal here
      
	thiscvg = args.cvg;                         % associated convergence
        thismaxit = args.maxit+previt;              % associated thismaxit
      
      	if ~isempty(parinit)
     	[parinit,xml,nbit] = valxem1(args.x,args.miss,obs,parinit,...
				     args.parknown,...
				     thismodel,thiscvg,thismaxit);
	end

        if isempty(parinit)		% failure ?
		fprintf(1,'estimation failed \n');
          nbfail = nbfail + 1;		% increase number of failure
          itrial = itrial - 1;		% this trial again
          % break				% go to loop of trials
        else
          nbfail = 0;			% no failure for this trial
        end

        totalit = totalit + nbit;		% total used iteration number

        switch args.saveit				% update previt
          case 'yes', previt = thismaxit - nbit;	% unused previous 
	                          % iteration number to save for the next algo
          case 'no',				% no saving
          otherwise fprintf(1,'unknown saveit');		% error
        end

      if (~isempty(xml) & (xml > bestxml))  % this run is the best until now ?
        bestxml = xml;			    % update xml
        bestpar(ikmod).xml = xml;
        bestpar(ikmod).p = parinit.p;	    % and parameters
        bestpar(ikmod).mu = parinit.mu;
        bestpar(ikmod).S = parinit.S;
	bestpar(ikmod).O = parinit.O;
	bestpar(ikmod).A = parinit.A;
        bestpar(ikmod).totalit = totalit;   % and total used iteration number
      end

      fprintf(1,'\n');


    end % loop on the number of trials

  end % loop on the model

end % loop on the number of clusters
