function [par,xml,nbit] = valxem1(x,miss,obs,parinit,parknown,model,cvg,maxit)

% [par,xml,nbit] = valxem1(x,miss,obs,parinit,parknown,model,cvg,maxit)
% One EM algorithm in the normal or exponential case with unobserved data
% input:
%	x		data: value {N} x [T_N,d] (cell)
%	miss		index of missing values: {N} x [m_N] (cell)
%       obs     	index of observed values: {N} x [m_n] (cell)
%	parinit		initial parameters (structure)
%	-.p		proportions [k]
%	-.mu		centers [k,d]
%	-.S		variance matrices [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A		transition matrix [k,k]
%	parknown	known parameters (structure)
%	-.p		'unknown' or proportions [k]
%	-.mu		'unknown' or centers [k,d]
%	-.S		'unknown' or variance matrices [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A   		'unknown' or transition matrix [k,k]
%	model		model and number of clusters (structure)
%	-.model		model: 'plkCk' or value cell of strings [nbmod]
%	-.k		number of clusters: n^0.3 or value [nbk]
%	-.A		type of transition matrix 
%	-.fam		family of conditionnal law : 'normal', 'exp' or 'np'
%	cvg		convergence 'xml', 'maxit' or 'xmlORmaxit'
%	maxit		maximum iterations number
% output:
%	par		estimated parameters (structure)
%	-.p		proportions [k]
%	-.mu		centers [k,d]
%	-.S		variance matrices [d,d,k]
%       -.O             observation matrix ('np' family) [k,v]
%	-.A 		transition matrix [k,k]
%	xml		loglikelihood 
%	nbit		used iteration number
% note:	par is empty if no convergence
%

k = length(parinit.p);		% number of clusters
N = length(x);                  % sample size
epscvg = 1e-6;			% epsilon for convergence

% modify convergence mode
% -----------------------

% em: 'partition' is replaced by 'xml'
if strcmp(cvg,'partition')
      cvg = 'xml';
end

fprintf(1,'[%s','em');

% initializations
% ---------------

cvgok = 0;			% convergence flag
nbit = 0;			% iteration number
told = NaN;			% partition
xmlold = -Inf;			% criterion

par = parinit;			% par set to initial par
if ~strcmp(parknown.p,'unknown'),
  par.p = parknown.p; end % par set to known par
if ~strcmp(parknown.mu,'unknown'), par.mu = parknown.mu; end
if ~strcmp(parknown.S,'unknown'), par.S = parknown.S; end
if ~strcmp(parknown.O,'unknown'), par.O = parknown.O; end
if ~strcmp(parknown.A,'unknown'), par.A = parknown.A; end

if strcmp(model.fam,'normal')
     % badly conditioned matrix: exit algorithm
     % ------------------------------------------

     if Sbadcond(par.S)
         par = []; xml = [];
	 fprintf(1,'.badly conditioned variance matrix');
     return
     end
end

% iterations
% ----------

while ~cvgok

  nbit = nbit + 1;		% one more iteration

  if ~mod(nbit,100)
    fprintf(1,'.%d',nbit);	% display every 100 iteration number
  end

  % step E : conditional probabilities
  % ----------------------------------

  [f,logf] = valxpdf(x,miss,par,model);        % density
			   % density
       for n = 1:N
            if (~all(f{n}))
                par = [];
		xml = -Inf;
		fprintf(1,' At least one element of sample has conditionnal probability zero');
		return
	    end	
	end
	    
    % em: no specific step 
    % -------------------- 
 
      [condalpha,probx] = condforward(f,par);     % forward algorithm 

      ml = chainml(probx);                         % loglikelihood of par
      condbeta = condbackward(probx,f,par);          % backward algorithm

      [z_zsuiv,z] = mcondprob(condalpha,condbeta,probx,par,f);
                   % conditional probabilities

      xml = sum(ml);

  % step M: update parameters 
  % ------------------------- 
 
   if isfield(par,'mu')  
   	muold = par.mu;                                      % current centers 
   end

  if strcmp(parknown.mu,'unknown') & ...
	(strcmp(model.fam,'normal') | strcmp(model.fam,'exp'))
    	
	[totalz,par.mu] = estimuv(x,miss,obs,z,par);	  % centers if unknown 
        if isempty(par.mu)
             par = []; xml = [];   
             fprintf(1,'can''t estimate centers\n');
             return 
        end
   elseif (strcmp(model.fam,'normal') | strcmp(model.fam,'exp'))
         par.mu = parknown.mu;
         d = size(x,2);	% dimension of data
         
         totalz = sum(z,2);

         if ~all(totalz)
                  fprintf(1,'\n at least one state unused; can''t estimate centers\n');
                 par.mu = [];
		 par = []; xml = [];  
	 end
     else
    	 par.mu = 'none';
     end

  if strcmp(parknown.O,'unknown') & strcmp(model.fam,'np')
    	
	[totalz,par.O] = estimOv(x,miss,obs,z,par);
	% observation matrices if unknown
        if isempty(par.O)
             par = []; xml = [];   
             fprintf(1,'can''t estimate observation matrix\n');
             return 
        end
   elseif strcmp(model.fam,'np')
         par.O = parknown.O;
         d = 1;	% dimension of data
         
         totalz = sum(z,2);

         if ~all(totalz)
                  fprintf(1,'\n at least one state unused; can''t estimate observation matrix\n');
                 par.O = [];
		 par = []; xml = [];  
	  end
     end

		 
  if (strcmp(parknown.S,'unknown')  & strcmp(model.fam,'normal'))
  	par.S = estimSv(x,miss,obs,z,model.model,totalz,muold,par);	
	% variance matrices if unknown       
	if isempty(par.S)
             par = []; xml = [];   
             fprintf(1,'can''t estimate covariance matrices\n');
             return 
        end
  end 

  if strcmp(parknown.A,'unknown') 
       par.A = estimAc(z,z_zsuiv);
       % transition matrix if unknown 
       if isempty(par.A)
             par = []; xml = [];   
             fprintf(1,'can''t estimate transition matrix\n');
             return 
       end
  end   
  if strcmp(parknown.p,'unknown') 
       par.p = estimpc(z,model.model,par.A);
       % proportions if unknown        
       if isempty(par.p)
             par = []; xml = [];   
             fprintf(1,'can''t estimate proportions\n');
             return   
       end
  end   

  if strcmp(model.fam,'normal')

     % badly conditioned matrix: exit algorithm 
     % ------------------------------------------ 
	     
     if Sbadcond(par.S) 
         par = []; xml = []; 
	 fprintf(1,'.badly conditioned variance matrix\n'); 
	 return 
     end 
  end

  % convergence ? 
  % ------------- 
 
  switch cvg 
 
    case 'maxit', cvgok = (nbit >= maxit);	% stop with iteration number 
    case 'xml',				% stop with criterion stability 
      cvgok = (abs(xml-xmlold) <= epscvg) ;	% xml stability ? 
      xmlold = xml; 
    case 'xmlORmaxit'			% stop with xml stability or maxit 
      cvgok = (nbit >= maxit) | (abs(xml-xmlold) <= epscvg); 
      xmlold = xml;     
    otherwise par = []; xml = []; fprintf(1,'.unknown convergence'); return 
 
  end 
 
  cvgok = cvgok | (k == 1);			% if one cluster, cvg is on 
 
end % while ~cvgok 

% display the iteration number 
% ---------------------------- 
 
if mod(nbit,100) 
  fprintf(1,'.%d',nbit);			% iteration number 
end 
fprintf(1,']'); 
 
