function [parout,obs] = valxpinit(parin,k,d,x,miss,A)

% [parout,obs] = valxpinit(parin,k,d,x,miss,A)
% Initialize normal mixture parameters from parameters specifications.
% input:
%	parin	parameters (structure)
%	-.p	'equal', 'random' or proportions values [k]
%	-.mu	'random' or centers value [k,d]
%	-.S	'identity', 'rand. var.', 'var. x' or variance matrices 
%                value [d,d,k]
%       -.A     'random' or transition matrix value [k,k]
%       -.O     observation matrix ('np' family) [k,v]
%	k	number of clusters
%	d	space dimension
%	x	data (cell) {N} x [nk,d]
%		only supplied if parin.mu is 'random')
%       miss	index of missing values: {N} x [m_n] (cell)	
%       A       type of transition matrix : 'full', 'utriangular' or 
%               'udiagonal'
% output:
%	parout	parameters (structure)
%	-.p	proportions value [k]
%	-.mu	centers value [k,d]
%	-.S	variance matrices value [d,d,k]
%       -.O     observation matrix ('np' family) [k,v]
%       -.A     transition matrix value [k,k]
%       obs     index of observed values: {N} x [m_n] (cell)

N = length(x);                  % sample size
for n = 1:N,
      obs{n} = setdiff([1:size(x{n},1)],miss{n});     
      % index of observed values
end

size_allx = 0;


% centers
% -------

parout.mu = parin.mu;
if isstr(parin.mu)
  switch parin.mu

    case 'random',
      N = length(x);
      for n = 1:N 	
	nk = size(x{n}(obs{n},:),1);		% length of nth sequence
	allx(size_allx + 1:size_allx + nk,:) = x{n}(obs{n},:);
	size_allx = size_allx + nk;
	if (size_allx > k)
		break;
	end;
       end;
       if k > size_allx , 
	 disp('mu: more centers than data'),parout= [];return,end
       
      permute = randperm(size_allx);		% permutation of integers 1:N
      parout.mu = allx(permute(1:k),:);	
	% select the k 1st x of thispermutation

   case 'none',
      parout.mu = ones(k,d);
         % the value of parout.mu will not be used anyway

    otherwise disp('mu: unknown value'),parout= [];return

  end
end

% transition matrix
% ------------------

parout.A = parin.A;
if isstr(parin.A)
  switch parin.A

    case 'random',
	parout.A = rand(k,k);
        parout.A = parout.A .* ...
                (ones(k,k)-tril(ones(k,k),A(1))-triu(ones(k,k),A(2)));
	for i = 1:k
		parout.A(i,:) = parout.A(i,:) / sum(parout.A(i,:),2);
	end;

    otherwise disp('A: unknown value'),parout= [];return

  end
end

% proportions
% -----------

parout.p = parin.p;
if isstr(parin.p)
  switch parin.p

    case 'equal', parout.p = ones(1,k) / k;

    case 'random'
      kunif = rand(1,k);		% k uniform numbers
      parout.p = kunif / sum(kunif);	% normalize

    case 'stationnary'
      [V,D] = eig((parout.A)');
      D = (diag(D) >= 0.999);               % locates the eigenvalue = 1      
      if ~(sum(D) == 1)
	     parout = []; 
    	     fprintf(1,'unique stationnary distribution does not exist'); 
    	     return 
	else
		V = V * D;              % eigenvector
                parout.p = (V / sum(V,1))';
	end
    otherwise disp('p: unknown value'),parout= [];return 

  end
end


% variance matrices
% -----------------

parout.S = parin.S;
if isstr(parin.S)
  switch parin.S

    case 'identity', parout.S = repmat(eye(d),[1,1,k]);

    case 'rand. var.'		% random volume for each variance matrix

      rndvar = 10*rand(k,1);
      parout.S = repmat(eye(d),[1,1,k]);
      for ik = 1:k
        parout.S(:,:,ik) = rndvar(ik) * parout.S(:,:,ik);
      end

    case 'var. x'			% same diagonal matrices 
                                        % with variances of sample
      if (size_allx == 0)
           N = length(x);
	   for n = 1:N 	
                 nk = size(x{n}(obs{n},:),1);	      % length of nth sequence
	         allx(size_allx + 1:size_allx + nk,:) = x{n}(obs{n},:);
                 size_allx = size_allx + nk;
                 if (size_allx > k)
	        	break;
		 end;
	    end;
       end;
	    
       if (k > size_allx)
	        disp('S: more  variance matrices than data'),parout= [];
                return;
       end

       parout.S = repmat(diag(var(allx)),[1 1 k]);

   case 'none',
      parout.S = ones(d,d,k);
         % the value of parout.S will not be used anyway

    otherwise disp('S: unknown value'),parout= [];return

  end
end

% observation matrices
% -----------------

parout.O = parin.O;
if isstr(parin.O)
  v = max(x{1}(obs{1},1));   % number of observations
  for n = 2:N
     v = max(v, max(x{n}(obs{n},1))); 
  end;

  switch parin.O

    case 'random'		% random observation matrix
	parout.O = rand(k,v);
	for i = 1:k
		parout.O(i,:) = parout.O(i,:) / sum(parout.O(i,:),2);
	end;


    case 'none',
       parout.O = ones(k,v);
         % the value of parout.O will not be used anyway

    otherwise disp('O: unknown value'),parout= [];return

  end

end

  
  
